@@ -30,6 +30,7 @@ typedef struct _GtkHTMLEditPropertiesDialog   GtkHTMLEditPropertiesDialog;
 
 #include <gtkhtml.h>
 #include <bonobo.h>
+#include "htmlgdkpainter.h"
 #include "search.h"
 #include "replace.h"
 #include "image.h"
@@ -39,8 +40,11 @@ typedef struct _GtkHTMLEditPropertiesDialog   GtkHTMLEditPropertiesDialog;
 #include "Spell.h"
 
 struct _GtkHTMLControlData {
-	GtkHTML   *html;
-	GtkWidget *vbox, *cpicker, *combo;
+	GtkHTML    *html;
+	GtkWidget  *vbox;
+	GtkWidget  *cpicker;
+	GtkWidget  *combo;
+	GtkWidget  *paragraph_option;
 
 	GtkHTMLEditPropertiesDialog   *properties_dialog;
 	GList                         *properties_types;
@@ -49,6 +53,11 @@ struct _GtkHTMLControlData {
 	GtkHTMLSearchDialog     *search_dialog;
 	GtkHTMLReplaceDialog    *replace_dialog;
 
+	/* html/plain mode settings */
+	gboolean format_html;
+	HTMLGdkPainter *gdk_painter;
+	HTMLGdkPainter *plain_painter;
+
 	/* object from last button press event */
 	HTMLObject *obj;
 
@@ -75,7 +84,8 @@ struct _GtkHTMLControlData {
 
 	GNOME_Spell_Dictionary  dict;
 	BonoboObjectClient     *dict_client;
-	EditorEngine       *editor_bonobo_engine;
+	EditorEngine           *editor_bonobo_engine;
+	BonoboControl          *control;
 };
 
 GtkHTMLControlData * gtk_html_control_data_new       (GtkHTML *html, GtkWidget *vbox);
