@@ -37,11 +37,17 @@
 #include "Editor.h"
 
 #include "gtkhtml.h"
+#include "gtkhtml-properties.h"
 #include "htmlcursor.h"
 #include "htmlengine.h"
 #include "htmlengine-edit.h"
 #include "htmlengine-edit-selection-updater.h"
 #include "htmlselection.h"
+#include "htmlfontmanager.h"
+#include "htmlsettings.h"
+#include "htmlpainter.h"
+#include "htmlplainpainter.h"
+
 
 #include "engine.h"
 #include "menubar.h"
@@ -93,6 +99,9 @@ set_frame_cb (BonoboControl *control,
 
 	control_data = (GtkHTMLControlData *) data;
 
+	if (bonobo_control_get_control_frame (control) == CORBA_OBJECT_NIL)
+		return;
+
 	remote_ui_container = bonobo_control_get_remote_ui_container (control);
 	ui_component = bonobo_control_get_ui_component (control);
 	bonobo_ui_component_set_container (ui_component, remote_ui_container);
@@ -339,6 +348,113 @@ destroy_control_data_cb (GtkObject *control, GtkHTMLControlData *cd)
 	gtk_main_quit ();
 }
 
+static void
+editor_init_painters (GtkHTMLControlData *cd)
+{	
+	GtkHTMLClassProperties *prop;
+	GtkHTML *html;
+
+	g_return_if_fail (cd != NULL);
+
+	html = cd->html;
+	prop = GTK_HTML_CLASS (GTK_OBJECT (html)->klass)->properties;
+	
+	if (!cd->plain_painter) {
+		cd->plain_painter = HTML_GDK_PAINTER (html_plain_painter_new (TRUE));
+		html_font_manager_set_default (&HTML_PAINTER (cd->plain_painter)->font_manager,
+					       prop->font_var,      prop->font_fix,
+					       prop->font_var_size, prop->font_fix_size);
+		
+		html_colorset_add_slave (html->engine->settings->color_set, 
+					 HTML_PAINTER (cd->plain_painter)->color_set);
+
+		cd->gdk_painter = HTML_GDK_PAINTER (html->engine->painter);
+
+		/* the plain painter starts with a ref */
+		gtk_object_ref (GTK_OBJECT (cd->gdk_painter));
+	}	
+}
+
+static void
+editor_set_format (GtkHTMLControlData *cd, gboolean format_html)
+{
+	HTMLGdkPainter *p, *old_p;
+	GtkHTML *html;
+
+	g_return_if_fail (cd != NULL);
+	
+	editor_init_painters (cd);
+	
+	html = cd->html;
+	cd->format_html = format_html;
+
+	if (format_html) {
+		p = cd->gdk_painter;
+		old_p = cd->plain_painter;
+	} else {
+		p = cd->plain_painter;
+		old_p = cd->gdk_painter;
+	}		
+
+	toolbar_update_format (cd);
+	menubar_update_format (cd);
+
+	if (html->engine->painter != (HTMLPainter *)p) {
+		html_gdk_painter_unrealize (old_p);
+		if (html->engine->window)
+			html_gdk_painter_realize (p, html->engine->window);
+
+		html_engine_set_painter (html->engine, HTML_PAINTER (p), 
+					 html->engine->width);
+		
+		html_engine_draw (html->engine, 0, 0, html->engine->width, html->engine->height);
+	}
+		
+}
+
+enum {
+	PROP_EDIT_HTML
+} EditorControlProps;
+
+static void
+editor_get_prop (BonoboPropertyBag *bag,
+		 BonoboArg         *arg,
+		 guint              arg_id,
+		 CORBA_Environment *ev,
+		 gpointer           user_data)
+{
+	GtkHTMLControlData *cd = user_data;
+	
+	switch (arg_id) {
+	case PROP_EDIT_HTML:
+		BONOBO_ARG_SET_BOOLEAN (arg, cd->format_html);
+		break;
+	default:
+		bonobo_exception_set (ev, ex_Bonobo_PropertyBag_NotFound);
+		break;
+	}
+}
+
+static void
+editor_set_prop (BonoboPropertyBag *bag,
+		 const BonoboArg   *arg,
+		 guint              arg_id,
+		 CORBA_Environment *ev,
+		 gpointer           user_data)
+{
+	GtkHTMLControlData *cd = user_data;
+	
+	g_warning ("set_prop");
+	switch (arg_id) {
+	case PROP_EDIT_HTML:
+		editor_set_format (cd, BONOBO_ARG_GET_BOOLEAN (arg));
+		break;
+	default:
+		bonobo_exception_set (ev, ex_Bonobo_PropertyBag_NotFound);
+		break;
+	}
+}
+
 static void
 editor_control_construct (BonoboControl *control, GtkWidget *vbox)
 {
@@ -346,6 +462,8 @@ editor_control_construct (BonoboControl *control, GtkWidget *vbox)
 	BonoboPersistStream *stream_impl;
 	BonoboPersistFile *file_impl;
 	GtkWidget *html_widget;
+	BonoboPropertyBag  *pb;
+	BonoboArg          *def;
 
 	active_controls++;
 
@@ -361,7 +479,8 @@ editor_control_construct (BonoboControl *control, GtkWidget *vbox)
 	/* HTMLEditor::Engine */
 
 	control_data->editor_bonobo_engine = editor_engine_new (GTK_HTML (html_widget));
-	bonobo_object_add_interface (BONOBO_OBJECT (control), BONOBO_OBJECT (control_data->editor_bonobo_engine));
+	bonobo_object_add_interface (BONOBO_OBJECT (control), 
+				     BONOBO_OBJECT (control_data->editor_bonobo_engine));
 
 	/* Bonobo::PersistStream */
 
@@ -373,6 +492,31 @@ editor_control_construct (BonoboControl *control, GtkWidget *vbox)
 	file_impl = persist_file_impl_new (GTK_HTML (html_widget));
 	bonobo_object_add_interface (BONOBO_OBJECT (control), BONOBO_OBJECT (file_impl));
 
+	/* PropertyBag */
+	pb = bonobo_property_bag_new (editor_get_prop, editor_set_prop, control_data);
+	bonobo_control_set_properties (control, pb);
+
+	def = bonobo_arg_new (BONOBO_ARG_BOOLEAN);
+	BONOBO_ARG_SET_BOOLEAN (def, TRUE);
+
+	bonobo_property_bag_add (pb, "FormatHTML", PROP_EDIT_HTML,
+				 BONOBO_ARG_BOOLEAN, def,
+				 "Whether or not to edit in HTML mode", 
+				 0);
+
+	CORBA_free (def);
+	/*
+	def = bonobo_arg_new (BONOBO_ARG_STRING);
+	BONOBO_ARG_SET_STRING (def, "");
+
+	bonobo_property_bag_add (pb, "OnURL", PROP_EDIT_HTML,
+				 BONOBO_ARG_STRING, def,
+				 "The URL of the link the mouse is currently over", 
+				 0);
+	*/
+
+	bonobo_object_unref (BONOBO_OBJECT (pb));
+
 	/* Part of the initialization must be done after the control is
 	   embedded in its control frame.  We use the "set_frame" signal to
 	   handle that.  */
@@ -388,6 +532,8 @@ editor_control_construct (BonoboControl *control, GtkWidget *vbox)
 
 	gtk_signal_connect (GTK_OBJECT (html_widget), "button_press_event",
 			    GTK_SIGNAL_FUNC (html_button_pressed), control_data);
+
+	control_data->control = control;
 }
 
 static BonoboObject *
@@ -504,7 +650,7 @@ editor_api_event (GtkHTML *html, GtkHTMLEditorEventType event_type, GtkArg **arg
 	return gtk_retval;
 }
 
-static GtkEntry *
+static GtkWidget *
 editor_api_create_input_line (GtkHTML *html, gpointer data)
 {
 	GtkHTMLControlData *cd = (GtkHTMLControlData *) data;
@@ -514,7 +660,7 @@ editor_api_create_input_line (GtkHTML *html, gpointer data)
 	gtk_box_pack_end (GTK_BOX (cd->vbox), entry, FALSE, FALSE, 0);
 	gtk_widget_show (entry);
 
-	return GTK_ENTRY (entry);
+	return entry;
 }
 
 static void
