@@ -20,7 +20,7 @@
     Boston, MA 02111-1307, USA.
 */
 
-#include "config.h"
+#include <config.h>
 #include "replace.h"
 #include "dialog.h"
 #include "htmlengine.h"
