@@ -48,6 +48,7 @@ static struct {
 	const gchar *description;
 } paragraph_style_items[] = {
 	{ GTK_HTML_PARAGRAPH_STYLE_NORMAL, N_("Normal") },
+	{ GTK_HTML_PARAGRAPH_STYLE_PRE, N_("Preformat") },
 	{ GTK_HTML_PARAGRAPH_STYLE_H1, N_("Header 1") },
 	{ GTK_HTML_PARAGRAPH_STYLE_H2, N_("Header 2") },
 	{ GTK_HTML_PARAGRAPH_STYLE_H3, N_("Header 3") },
@@ -55,7 +56,6 @@ static struct {
 	{ GTK_HTML_PARAGRAPH_STYLE_H5, N_("Header 5") },
 	{ GTK_HTML_PARAGRAPH_STYLE_H6, N_("Header 6") },
 	{ GTK_HTML_PARAGRAPH_STYLE_ADDRESS, N_("Address") },
-	{ GTK_HTML_PARAGRAPH_STYLE_PRE, N_("Pre") },
 	{ GTK_HTML_PARAGRAPH_STYLE_ITEMDIGIT, N_("List item (digit)") },
 	{ GTK_HTML_PARAGRAPH_STYLE_ITEMDOTTED, N_("List item (unnumbered)") },
 	{ GTK_HTML_PARAGRAPH_STYLE_ITEMROMAN, N_("List item (roman)") },
@@ -97,6 +97,32 @@ paragraph_style_menu_item_activated_cb (GtkWidget *widget,
 	gtk_html_set_paragraph_style (html, style);
 }
 
+static void
+paragraph_style_menu_item_update (GtkWidget *widget, gpointer format_html)
+{
+	GtkHTMLParagraphStyle style;
+	gint sensitive;
+
+	style = GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (widget), "paragraph_style_value"));
+	
+	sensitive = (format_html 
+		|| style == GTK_HTML_PARAGRAPH_STYLE_NORMAL
+		|| style == GTK_HTML_PARAGRAPH_STYLE_PRE);
+
+	gtk_widget_set_sensitive (widget, sensitive);	
+}
+
+static void
+paragraph_style_option_menu_set_mode (GtkWidget *option_menu, gboolean format_html)
+{
+	GtkWidget *menu;
+	
+	menu = gtk_option_menu_get_menu (GTK_OPTION_MENU (option_menu));
+	gtk_container_forall (GTK_CONTAINER (menu), 
+			      paragraph_style_menu_item_update, 
+			      GINT_TO_POINTER (format_html));
+}
+
 static GtkWidget *
 setup_paragraph_style_option_menu (GtkHTML *html)
 {
@@ -206,13 +232,50 @@ color_changed (GtkWidget *w, GdkColor *gdk_color, GtkHTMLControlData *cd)
 		html_color_unref (color);
 }
 
+static void
+unset_focus (GtkWidget *w, gpointer data)
+{
+	GTK_WIDGET_UNSET_FLAGS (w, GTK_CAN_FOCUS);
+}
+
+inline static void
+set_color_combo (GtkHTML *html, GtkHTMLControlData *cd)
+{
+	color_combo_set_color (COLOR_COMBO (cd->combo),
+			       &html_colorset_get_color_allocated (html->engine->painter, HTMLTextColor)->color);
+}
+
+static void
+realize_engine (GtkHTML *html, GtkHTMLControlData *cd)
+{
+	set_color_combo (html, cd);
+	gtk_signal_disconnect_by_func (GTK_OBJECT (html), realize_engine, cd);
+}
+
+static void
+load_done (GtkHTML *html, GtkHTMLControlData *cd)
+{
+	if (GTK_WIDGET_REALIZED (cd->html))
+		set_color_combo (html, cd);
+	else
+		gtk_signal_connect (GTK_OBJECT (cd->html), "realize", realize_engine, cd);
+}
+
 static GtkWidget *
 setup_color_combo (GtkHTMLControlData *cd)
 {
-	cd->combo = color_combo_new (NULL, _("Automatic"),
-				     &html_colorset_get_color (cd->html->engine->settings->color_set,
-							       HTMLTextColor)->color, "toolbar_text");
+	HTMLColor *color;
+
+	color = html_colorset_get_color (cd->html->engine->settings->color_set, HTMLTextColor);
+	if (GTK_WIDGET_REALIZED (cd->html))
+		html_color_alloc (color, cd->html->engine->painter);
+	else
+		gtk_signal_connect (GTK_OBJECT (cd->html), "realize", realize_engine, cd);
+        gtk_signal_connect (GTK_OBJECT (cd->html), "load_done", GTK_SIGNAL_FUNC (load_done), cd);
+
+	cd->combo = color_combo_new (NULL, _("Automatic"), &color->color, "toolbar_text");
 	GTK_WIDGET_UNSET_FLAGS (cd->combo, GTK_CAN_FOCUS);
+	gtk_container_forall (GTK_CONTAINER (cd->combo), unset_focus, NULL);
         gtk_signal_connect (GTK_OBJECT (cd->combo), "changed", GTK_SIGNAL_FUNC (color_changed), cd);
 
 	gtk_widget_show_all (cd->combo);
@@ -486,9 +549,11 @@ create_style_toolbar (GtkHTMLControlData *cd)
 
 	gtk_widget_show_all (hbox);
 
+	cd->paragraph_option = setup_paragraph_style_option_menu (cd->html),
 	gtk_toolbar_prepend_widget (GTK_TOOLBAR (cd->toolbar_style),
-				    setup_paragraph_style_option_menu (cd->html),
+				    cd->paragraph_option,
 				    NULL, NULL);
+
 	gtk_toolbar_prepend_widget (GTK_TOOLBAR (cd->toolbar_style),
 				    setup_font_size_option_menu (cd),
 				    NULL, NULL);
@@ -525,6 +590,22 @@ create_style_toolbar (GtkHTMLControlData *cd)
 	return hbox;
 }
 
+static void
+toolbar_item_update_sensitivity (GtkWidget *widget, gpointer data)
+{
+	GtkHTMLControlData *cd = (GtkHTMLControlData *)data;
+	
+	if (widget != cd->paragraph_option)
+		gtk_widget_set_sensitive (widget, cd->format_html);
+}
+
+void
+toolbar_update_format (GtkHTMLControlData *cd)
+{
+	gtk_container_forall (GTK_CONTAINER (cd->toolbar_style), toolbar_item_update_sensitivity, cd);
+	paragraph_style_option_menu_set_mode (cd->paragraph_option, cd->format_html);
+}
+
 
 GtkWidget *
 toolbar_style (GtkHTMLControlData *cd)
