@@ -20,6 +20,13 @@
     Boston, MA 02111-1307, USA.
 */
 
+#include <config.h>
+#include <gal/widgets/e-scroll-frame.h>
+
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <fcntl.h>
+#include <gnome.h>
 #include "utils.h"
 
 GtkWidget *
@@ -62,3 +69,47 @@ color_table_new (GtkSignalFunc f, gpointer data)
 
 	return table;
 }
+
+static void
+url_requested (GtkHTML *html, const gchar *url, GtkHTMLStream *handle)
+{
+	GtkHTMLStreamStatus status;
+	gint fd;
+
+	if (!strncmp (url, "file:", 5))
+		url += 5;
+
+	fd = open (url, O_RDONLY);
+	status = GTK_HTML_STREAM_OK;
+	if (fd != -1) {
+		ssize_t size;
+		void *buf = alloca (1 << 7);
+		while ((size = read (fd, buf, 1 << 7))) {
+			if (size == -1) {
+				status = GTK_HTML_STREAM_ERROR;
+				break;
+			} else
+				gtk_html_write (html, handle, (const gchar *) buf, size);
+		}
+	} else
+		status = GTK_HTML_STREAM_ERROR;
+	gtk_html_end (html, handle, status);
+}
+
+GtkWidget *
+sample_frame (GtkHTML **html)
+{
+	GtkWidget *frame, *scroll_frame;
+
+	frame = gtk_frame_new (_("Sample"));
+	*html = GTK_HTML (gtk_html_new ());
+	scroll_frame = e_scroll_frame_new (NULL, NULL);
+	e_scroll_frame_set_shadow_type (E_SCROLL_FRAME (scroll_frame), GTK_SHADOW_IN);
+	gtk_container_border_width (GTK_CONTAINER (scroll_frame), 3);
+	e_scroll_frame_set_policy (E_SCROLL_FRAME (scroll_frame), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
+	gtk_container_add (GTK_CONTAINER (scroll_frame), GTK_WIDGET (*html));
+	gtk_container_add (GTK_CONTAINER (frame), scroll_frame);
+	gtk_signal_connect (GTK_OBJECT (*html), "url_requested", url_requested, NULL);
+
+	return frame;
+}
