@@ -84,6 +84,9 @@ typedef enum {
 	GTK_HTML_COMMAND_INDENT_DEC,
 	GTK_HTML_COMMAND_INDENT_PARAGRAPH,
 
+	GTK_HTML_COMMAND_BREAK_AND_FILL_LINE,
+	GTK_HTML_COMMAND_SPACE_AND_FILL_LINE,
+
 	GTK_HTML_COMMAND_PARAGRAPH_STYLE_NORMAL,
 	GTK_HTML_COMMAND_PARAGRAPH_STYLE_H1,
 	GTK_HTML_COMMAND_PARAGRAPH_STYLE_H2,
@@ -128,7 +131,32 @@ typedef enum {
 	GTK_HTML_COMMAND_PROPERTIES_DIALOG,
 
 	GTK_HTML_COMMAND_CURSOR_FORWARD,
-	GTK_HTML_COMMAND_CURSOR_BACKWARD
+	GTK_HTML_COMMAND_CURSOR_BACKWARD,
+
+	GTK_HTML_COMMAND_INSERT_TABLE_1_1,
+
+	GTK_HTML_COMMAND_TABLE_INSERT_COL_AFTER,
+	GTK_HTML_COMMAND_TABLE_INSERT_COL_BEFORE,
+	GTK_HTML_COMMAND_TABLE_INSERT_ROW_AFTER,
+	GTK_HTML_COMMAND_TABLE_INSERT_ROW_BEFORE,
+	GTK_HTML_COMMAND_TABLE_DELETE_COL,
+	GTK_HTML_COMMAND_TABLE_DELETE_ROW,
+
+	GTK_HTML_COMMAND_TABLE_CELL_INC_CSPAN,
+	GTK_HTML_COMMAND_TABLE_CELL_DEC_CSPAN,
+	GTK_HTML_COMMAND_TABLE_CELL_INC_RSPAN,
+	GTK_HTML_COMMAND_TABLE_CELL_DEC_RSPAN,
+
+	GTK_HTML_COMMAND_TABLE_CELL_JOIN_LEFT,
+	GTK_HTML_COMMAND_TABLE_CELL_JOIN_RIGHT,
+	GTK_HTML_COMMAND_TABLE_CELL_JOIN_UP,
+	GTK_HTML_COMMAND_TABLE_CELL_JOIN_DOWN,
+
+	GTK_HTML_COMMAND_TABLE_BORDER_WIDTH_INC,
+	GTK_HTML_COMMAND_TABLE_BORDER_WIDTH_DEC,
+	GTK_HTML_COMMAND_TABLE_BORDER_WIDTH_ZERO,
+
+	GTK_HTML_COMMAND_TEXT_SET_DEFAULT_COLOR,
 } GtkHTMLCommandType;
 
 typedef enum {
