@@ -29,14 +29,16 @@
 
 #include <gnome.h>
 
+#include "htmlengine-edit.h"
 #include "htmlengine-edit-clueflowstyle.h"
 #include "htmlengine-edit-cut-and-paste.h"
 #include "htmlengine-edit-fontstyle.h"
 #include "htmlengine-edit-rule.h"
 #include "htmlengine-edit-movement.h"
 #include "htmlengine-edit-cursor.h"
+#include "htmlengine-edit-table.h"
 #include "htmlengine-edit-text.h"
-#include "htmlengine-edit.h"
+#include "htmlengine-edit-selection-updater.h"
 #include "htmlengine-print.h"
 #include "htmlengine-save.h"
 #include "htmlsettings.h"
@@ -44,6 +46,7 @@
 #include "htmlselection.h"
 #include "htmlcursor.h"
 
+#include "gtkhtml.h"
 #include "gtkhtml-embedded.h"
 #include "gtkhtml-im.h"
 #include "gtkhtml-keybinding.h"
@@ -79,6 +82,7 @@ enum {
 	INSERTION_FONT_STYLE_CHANGED,
 	INSERTION_COLOR_CHANGED,
 	SIZE_CHANGED,
+	IFRAME_CREATED,
 	/* keybindings signals */
 	SCROLL,
 	CURSOR_MOVE,
@@ -219,6 +223,8 @@ update_styles (GtkHTML *html)
 	HTMLEngine *engine;
 	guint indentation;
 
+	/* printf ("update_styles called\n"); */
+
 	if (! html_engine_get_editable (html->engine))
 		return;
 
@@ -247,9 +253,9 @@ update_styles (GtkHTML *html)
 
 	if (html_engine_update_insertion_font_style (engine))
 		gtk_signal_emit (GTK_OBJECT (html), signals[INSERTION_FONT_STYLE_CHANGED], engine->insertion_font_style);
-
 	if (html_engine_update_insertion_color (engine))
 		gtk_signal_emit (GTK_OBJECT (html), signals[INSERTION_COLOR_CHANGED], engine->insertion_color);
+
 	/* TODO add insertion_url_or_targed_changed signal */
 	html_engine_update_insertion_url_and_target (engine);
 }
@@ -620,15 +626,17 @@ key_press_event (GtkWidget *widget,
 		 GdkEventKey *event)
 {
 	GtkHTML *html = GTK_HTML (widget);
-	gboolean retval;
+	gboolean retval, update = TRUE;
 	gint position = html->engine->cursor->position;
 
 	html->binding_handled = FALSE;
+	html->priv->update_styles = FALSE;
 	if (html->editor_bindings && html_engine_get_editable (html->engine))
 		gtk_binding_set_activate (html->editor_bindings, event->keyval, event->state, GTK_OBJECT (widget));
 	if (!html->binding_handled)
 		gtk_bindings_activate (GTK_OBJECT (widget), event->keyval, event->state);
 	retval = html->binding_handled;
+	update = html->priv->update_styles;
 
 	if (! retval
 	    && html_engine_get_editable (html->engine)
@@ -646,16 +654,19 @@ key_press_event (GtkWidget *widget,
 			html_engine_paste_text (html->engine, event->string, 1);
 		g_free (str);
 		retval = TRUE;
+		update = FALSE;
 	}
 
 	if (retval && html_engine_get_editable (html->engine))
 		html_engine_reset_blinking_cursor (html->engine);
 
 	if (retval) {
-		if (position != html->engine->cursor->position)
+		if (position != html->engine->cursor->position && update)
 			update_styles (html);
 	}
 
+	/* printf ("retval: %d\n", retval); */
+
 	return retval;
 }
 
@@ -802,8 +813,8 @@ on_object (GtkWidget *widget, GdkWindow *window, HTMLObject *obj)
 		}
 	} else {
 		if (html->pointer_url) {
-			html->pointer_url = NULL;
 			g_free (html->pointer_url);
+			html->pointer_url = NULL;
 			gtk_signal_emit (GTK_OBJECT (html), signals [ON_URL], NULL);
 		}
 		gdk_window_set_cursor (window, html->arrow_cursor);
@@ -993,10 +1004,14 @@ button_press_event (GtkWidget *widget,
 		case 1:
 			html->button1_pressed = TRUE;
 			if (html_engine_get_editable (engine)) {
+				if (html->allow_selection) {
+					if (event->state & GDK_SHIFT_MASK)
+						html_engine_set_mark (engine);
+				}
+
 				html_engine_jump_at (engine,
 						     x + engine->x_offset,
 						     y + engine->y_offset);
-				update_styles (html);
 			}
 
 			if (html->allow_selection) {
@@ -1015,8 +1030,10 @@ button_press_event (GtkWidget *widget,
 						html->selection_y1 = y + engine->y_offset;
 					}
 				}
-
 			}
+
+			if (html_engine_get_editable (engine))
+				update_styles (html);
 			break;
 		default:
 			break;
@@ -1230,7 +1247,8 @@ selection_received (GtkWidget *widget,
 		if (selection_data->type != GDK_SELECTION_TYPE_STRING) {
 			html_engine_paste_text (GTK_HTML (widget)->engine, 
 						selection_data->data,
-						(guint) selection_data->length);
+						(guint) unicode_strlen (selection_data->data, 
+									selection_data->length));
 		} else {
 			gchar *utf8 = NULL;
 			
@@ -1637,6 +1655,14 @@ class_init (GtkHTMLClass *klass)
 				GTK_SIGNAL_OFFSET (GtkHTMLClass, size_changed),
 				gtk_marshal_NONE__NONE,
 				GTK_TYPE_NONE, 0);
+	signals [IFRAME_CREATED] = 
+		gtk_signal_new ("iframe_created",
+				GTK_RUN_FIRST,
+				object_class->type,
+				GTK_SIGNAL_OFFSET (GtkHTMLClass, iframe_created),
+				gtk_marshal_NONE__POINTER,
+				GTK_TYPE_NONE, 1,
+				GTK_TYPE_HTML);
 
 	/* signals for keybindings */
 	signals [SCROLL] =
@@ -1747,8 +1773,6 @@ init (GtkHTML* html)
 	html->in_selection = FALSE;
 	html->button1_pressed = FALSE;
 
-	html->load_in_progress = TRUE;
-
 	html->priv = g_new0 (GtkHTMLPrivate, 1);
 	html->priv->idle_handler_id = 0;
 	html->priv->scroll_timeout_id = 0;
@@ -1890,32 +1914,35 @@ gtk_html_allow_selection (GtkHTML *html,
 }
 
 
-
-/* #define LOG_INPUT */
-#ifdef LOG_INPUT
-static FILE *log_file = NULL;
-static gint  log_num = 1;
-#endif
-
+/**
+ * gtk_html_begin:
+ * @html: the html widget to operate on.
+ * 
+ * Opens a new stream to load new content into the GtkHTML widget @html.
+ *
+ * Returns: a new GtkHTMLStream to store new content.
+ **/
 GtkHTMLStream *
 gtk_html_begin (GtkHTML *html)
 {
 	return gtk_html_begin_content (html, html->priv->content_type);
 }
 
+/**
+ * gtk_html_begin_content:
+ * @html: the html widget to operate on.
+ * @content_type: a string listing the type of content to expect on the stream.
+ *
+ * Opens a new stream to load new content of type @content_type into 
+ * the GtkHTML widget given in @html.
+ *
+ * Returns: a new GtkHTMLStream to store new content.
+ **/
 GtkHTMLStream *
 gtk_html_begin_content (GtkHTML *html, gchar *content_type)
 {
 	GtkHTMLStream *handle;
 
-#ifdef LOG_INPUT
-	gchar *fname = g_strdup_printf ("gtkhtml.log.%d", log_num);
-	log_num ++;
-	if (log_file)
-		fclose (log_file);
-	log_file = fopen (fname, "w+");
-	g_free (fname);
-#endif
 	g_return_val_if_fail (! gtk_html_get_editable (html), NULL);
 
 	handle = html_engine_begin (html->engine, content_type);
@@ -1924,41 +1951,53 @@ gtk_html_begin_content (GtkHTML *html, gchar *content_type)
 
 	html_engine_parse (html->engine);
 
-	html->load_in_progress = TRUE;
-
 	return handle;
 }
 
+/**
+ * gtk_html_write:
+ * @html: the GtkHTML widget the stream belongs to (unused)
+ * @handle: the GkHTMLStream to write to.
+ * @buffer: the data to write to the stream.
+ * @size: the length of data to read from @buffer
+ * 
+ * Writes @size bytes of @buffer to the stream pointed to by @stream.
+ **/
 void
 gtk_html_write (GtkHTML *html,
 		GtkHTMLStream *handle,
 		const gchar *buffer,
 		size_t size)
 {
-#ifdef LOG_INPUT
-	gint i;
-	for (i=0; i<size; i++)
-		fprintf (log_file, "%c", buffer [i]);
-#endif
 	gtk_html_stream_write (handle, buffer, size);
 }
 
+/**
+ * gtk_html_end:
+ * @html: the GtkHTML widget the stream belongs to.
+ * @handle: the GtkHTMLStream to close.
+ * @status: the GtkHTMLStreamStatus representing the state of the stream when closed.
+ *
+ * Close the GtkHTMLStream represented by @stream and notify @html that is 
+ * should not expect any more content from that stream.
+ **/
 void
 gtk_html_end (GtkHTML *html,
 	      GtkHTMLStream *handle,
 	      GtkHTMLStreamStatus status)
 {
-#ifdef LOG_INPUT
-	if (log_file)
-		fclose (log_file);
-	log_file = NULL;
-#endif
 	gtk_html_stream_close (handle, status);
-
-	html->load_in_progress = FALSE;
 }
 
 
+/**
+ * gtk_html_get_title:
+ * @html: The GtkHTML widget.
+ *
+ * Retrieve the title of the document currently loaded in the GtkHTML widget.
+ *
+ * Returns: the title of the current document
+ **/
 const gchar *
 gtk_html_get_title (GtkHTML *html)
 {
@@ -1972,6 +2011,15 @@ gtk_html_get_title (GtkHTML *html)
 }
 
 
+/**
+ * gtk_html_jump_to_anchor:
+ * @html: the GtkHTML widget.
+ * @anchor: a string containing the name of the anchor.
+ *
+ * Scroll the document display to show the HTML anchor listed in @anchor
+ *
+ * Returns: TRUE if the anchor is found, FALSE otherwise.
+ **/
 gboolean
 gtk_html_jump_to_anchor (GtkHTML *html,
 			 const gchar *anchor)
@@ -1995,21 +2043,34 @@ gtk_html_save (GtkHTML *html,
 	return html_engine_save (html->engine, receiver, data);
 }
 
+/**
+ * gtk_html_export:
+ * @html: the GtkHTML widget
+ * @content_type: the expected content_type 
+ * @receiver: 
+ * @user_data: pointer to maintain user state. 
+ *
+ * Export the current document into the content type given by @content_type,
+ * by calling the function listed in @receiver data becomes avaiable.  When @receiver is 
+ * called @user_data is passed in as the user_data parameter.
+ * 
+ * Returns: TRUE if the export was successfull, FALSE otherwise.
+ **/
 gboolean
 gtk_html_export (GtkHTML *html,
-		 const char *type,
+		 const char *content_type,
 		 GtkHTMLSaveReceiverFn receiver,
-		 gpointer data)
+		 gpointer user_data)
 {
 	g_return_val_if_fail (html != NULL, FALSE);
 	g_return_val_if_fail (GTK_IS_HTML (html), FALSE);
 	g_return_val_if_fail (receiver != NULL, FALSE);
 	
-	if (strcmp (type, "text/html") == 0) {
-		return html_engine_save (html->engine, receiver, data);
-	} else if (strcmp (type, "text/plain") == 0) {
+	if (strcmp (content_type, "text/html") == 0) {
+		return html_engine_save (html->engine, receiver, user_data);
+	} else if (strcmp (content_type, "text/plain") == 0) {
 		return html_engine_save_plain (html->engine, receiver,
-					       data);  
+					       user_data);  
 	} else {
 		return FALSE;
 	}
@@ -2491,6 +2552,7 @@ cursor_move (GtkHTML *html, GtkDirectionType dir_type, GtkHTMLCursorSkipType ski
 	}
 
 	html->binding_handled = TRUE;
+	html->priv->update_styles = TRUE;
 	gtk_html_edit_make_cursor_visible (html);
 }
 
@@ -2543,6 +2605,7 @@ move_selection (GtkHTML *html, GtkHTMLCommandType com_type)
 	}
 
 	html->binding_handled = TRUE;
+	html->priv->update_styles = TRUE;
 	gtk_html_edit_make_cursor_visible (html);
 }
 
@@ -2572,10 +2635,11 @@ command (GtkHTML *html, GtkHTMLCommandType com_type)
 		html->binding_handled = FALSE;
 	}
 
-	if (!html_engine_get_editable (e))
+	if (!html_engine_get_editable (e) || html->binding_handled)
 		return;
 
 	html->binding_handled = TRUE;
+	html->priv->update_styles = FALSE;
 
 	/* editable commands only */
 	switch (com_type) {
@@ -2715,6 +2779,12 @@ command (GtkHTML *html, GtkHTMLCommandType com_type)
 	case GTK_HTML_COMMAND_INDENT_PARAGRAPH:
 		html_engine_indent_pre_paragraph (e);
 		break;
+	case GTK_HTML_COMMAND_BREAK_AND_FILL_LINE:
+		html_engine_break_and_fill_line (e);
+		break;
+	case GTK_HTML_COMMAND_SPACE_AND_FILL_LINE:
+		html_engine_space_and_fill_line (e);
+		break;
 	case GTK_HTML_COMMAND_PARAGRAPH_STYLE_NORMAL:
 		gtk_html_set_paragraph_style (html, GTK_HTML_PARAGRAPH_STYLE_NORMAL);
 		break;
@@ -2793,6 +2863,39 @@ command (GtkHTML *html, GtkHTMLCommandType com_type)
 	case GTK_HTML_COMMAND_CURSOR_BACKWARD:
 		html_cursor_backward (html->engine->cursor, html->engine);
 		break;
+	case GTK_HTML_COMMAND_INSERT_TABLE_1_1:
+		html_engine_insert_table_1_1 (e);
+		break;
+	case GTK_HTML_COMMAND_TABLE_INSERT_COL_BEFORE:
+		html_engine_insert_table_column (e, FALSE);
+		break;
+	case GTK_HTML_COMMAND_TABLE_INSERT_COL_AFTER:
+		html_engine_insert_table_column (e, TRUE);
+		break;
+	case GTK_HTML_COMMAND_TABLE_DELETE_COL:
+		html_engine_delete_table_column (e);
+		break;
+	case GTK_HTML_COMMAND_TABLE_INSERT_ROW_BEFORE:
+		html_engine_insert_table_row (e, FALSE);
+		break;
+	case GTK_HTML_COMMAND_TABLE_INSERT_ROW_AFTER:
+		html_engine_insert_table_row (e, TRUE);
+		break;
+	case GTK_HTML_COMMAND_TABLE_DELETE_ROW:
+		html_engine_delete_table_row (e);
+		break;
+	case GTK_HTML_COMMAND_TABLE_BORDER_WIDTH_INC:
+		html_engine_table_set_border_width (e, 1, TRUE);
+		break;
+	case GTK_HTML_COMMAND_TABLE_BORDER_WIDTH_DEC:
+		html_engine_table_set_border_width (e, -1, TRUE);
+		break;
+	case GTK_HTML_COMMAND_TABLE_BORDER_WIDTH_ZERO:
+		html_engine_table_set_border_width (e, 0, FALSE);
+		break;
+	case GTK_HTML_COMMAND_TEXT_SET_DEFAULT_COLOR:
+		html_engine_set_color (e, NULL);
+		break;
 	default:
 		html->binding_handled = FALSE;
 	}
@@ -2988,6 +3091,8 @@ gtk_html_set_iframe_parent (GtkHTML *html, GtkWidget *parent)
 	g_assert (GTK_IS_HTML (parent));
 
 	html->iframe_parent = parent;
+	gtk_signal_emit (GTK_OBJECT (html_engine_get_top_html_engine (html->engine)->widget),
+			 signals [IFRAME_CREATED], html);
 }
 
 void
@@ -3084,3 +3189,13 @@ gtk_html_edit_make_cursor_visible (GtkHTML *html)
 
 	return rv;
 }
+
+gboolean
+gtk_html_build_with_gconf ()
+{
+#ifdef GTKHTML_HAVE_GCONF
+	return TRUE;
+#else
+	return FALSE;
+#endif
+}
