@@ -23,6 +23,7 @@
 #define _GTKHTML_H_
 
 #include <sys/types.h>
+#include <gtk/gtkbindings.h>
 #include <gtk/gtklayout.h>
 #include <libgnomeprint/gnome-print.h>
 
@@ -60,7 +61,6 @@ struct _GtkHTML {
 
 	guint in_selection : 1;
 	guint button1_pressed : 1;
-	guint load_in_progress : 1;
 
 	guint debug : 1;
 	guint allow_selection : 1;
@@ -97,6 +97,7 @@ struct _GtkHTMLClass {
 	void (* insertion_color_changed) (GtkHTML *html, GdkColor *color);
 
         void (* size_changed)       (GtkHTML *html);
+	void (* iframe_created)     (GtkHTML *html, GtkHTML *iframe);
 
 	/* keybindings signals */
 	void (* scroll)               (GtkHTML *html, GtkOrientation orientation, GtkScrollType scroll_type, gfloat position);
@@ -121,7 +122,7 @@ struct _GtkHTMLEditorAPI
 	GtkArg  * (* event)                   (GtkHTML *html, GtkHTMLEditorEventType event_type, GtkArg **args, gpointer data);
 
 	/* input line */
-	GtkEntry * (* create_input_line)      (GtkHTML *html, gpointer data);
+	GtkWidget * (* create_input_line)     (GtkHTML *html, gpointer data);
 };
 
 
@@ -242,5 +243,6 @@ gpointer  gtk_html_get_object_by_id              (GtkHTML     *html,
 void      gtk_html_editor_command                (GtkHTML     *html,
 						  const gchar *command_name);
 gboolean  gtk_html_edit_make_cursor_visible      (GtkHTML     *html);
-     
+
+gboolean  gtk_html_build_with_gconf              (void);
 #endif /* _GTKHTML_H_ */
