@@ -29,6 +29,7 @@
 #include "htmltext.h"
 #include "htmltextslave.h"
 #include "htmltable.h"
+#include "htmltablecell.h"
 #include "htmlclue.h"
 #include "htmlclueflow.h"
 #include "htmliframe.h"
@@ -106,7 +107,21 @@ gtk_html_debug_dump_table (HTMLObject *o,
 
 	for (r = 0; r < table->totalRows; r++) {
 		for (c = 0; c < table->totalCols; c++) {
-			gtk_html_debug_dump_tree (HTML_OBJECT (table->cells[r][c]), level + 1);
+			gtk_html_debug_dump_tree (HTML_OBJECT (table->cells[r][c]), level);
+		}
+	}
+
+}
+
+static void
+gtk_html_debug_dump_table_simple (HTMLObject *o, gint level)
+{
+	gint c, r;
+	HTMLTable *table = HTML_TABLE (o);
+
+	for (r = 0; r < table->totalRows; r++) {
+		for (c = 0; c < table->totalCols; c++) {
+			gtk_html_debug_dump_tree_simple (HTML_OBJECT (table->cells[r][c]), level);
 		}
 	}
 
@@ -120,9 +135,9 @@ gtk_html_debug_dump_object (HTMLObject *obj,
 	for (i = 0; i < level; i++)
 		g_print (" ");
 
-	g_print ("ObjectType: %s Pos: %d, %d, MinWidth: %d, Width: %d MaxWidth: %d Ascent %d Descent %d",
+	g_print ("ObjectType: %s Pos: %d, %d, MinWidth: %d, Width: %d PrefWidth: %d MaxWidth: %d Ascent %d Descent %d",
 		 html_type_name (HTML_OBJECT_TYPE (obj)),
-		 obj->x, obj->y, obj->min_width, obj->width, obj->max_width, obj->ascent, obj->descent);
+		 obj->x, obj->y, obj->min_width, obj->width, obj->pref_width, obj->max_width, obj->ascent, obj->descent);
 
 	if (HTML_OBJECT_TYPE (obj) == HTML_TYPE_CLUEFLOW)
 		g_print (" [%s, %d]",
@@ -141,7 +156,6 @@ gtk_html_debug_dump_object (HTMLObject *obj,
 			 sl_text);
 		g_free (sl_text);
 	}
-			
 
 	g_print ("\n");
 
@@ -160,7 +174,7 @@ gtk_html_debug_dump_object (HTMLObject *obj,
 	case HTML_TYPE_CLUEH:
 	case HTML_TYPE_CLUEV:
 	case HTML_TYPE_CLUEFLOW:
-		g_print ("Head: %p Tail: %p\n", HTML_CLUE (obj)->head, HTML_CLUE (obj)->tail);
+		/* g_print ("Head: %p Tail: %p\n", HTML_CLUE (obj)->head, HTML_CLUE (obj)->tail); */
 	case HTML_TYPE_CLUEALIGNED:
 	case HTML_TYPE_TABLECELL:
 		for (i = 0; i < level; i++) g_print (" ");
@@ -191,10 +205,13 @@ gtk_html_debug_dump_tree (HTMLObject *o,
 
 
 static void
-debug_word_width (HTMLText *t)
+debug_word_width (HTMLText *t, gint level)
 {
 	guint i;
 
+	for (i = 0; i < level; i++)
+		g_print ("\t");
+
 	printf ("words: %d | ", t->words);
 	for (i = 0; i < t->words; i ++)
 		printf ("%d ", t->word_width [i]);
@@ -214,7 +231,7 @@ dump_object_simple (HTMLObject *obj,
 		g_print ("%s `%s'\n",
 			 html_type_name (HTML_OBJECT_TYPE (obj)),
 			 HTML_TEXT (obj)->text);
-		debug_word_width (HTML_TEXT (obj));
+		debug_word_width (HTML_TEXT (obj), level);
 	} else if (HTML_OBJECT_TYPE (obj) == HTML_TYPE_TEXTSLAVE) {
 		HTMLTextSlave *slave = HTML_TEXT_SLAVE (obj);
 		gchar *text;
@@ -225,6 +242,12 @@ dump_object_simple (HTMLObject *obj,
 		g_print ("%s `%s'\n",
 			 html_type_name (HTML_OBJECT_TYPE (obj)),
 			 text);
+	} else if (HTML_OBJECT_TYPE (obj) == HTML_TYPE_TABLECELL) {
+		g_print ("%s %d,%d\n", html_type_name (HTML_OBJECT_TYPE (obj)),
+			 HTML_TABLE_CELL (obj)->row, HTML_TABLE_CELL (obj)->col);
+	} else if (HTML_OBJECT_TYPE (obj) == HTML_TYPE_TABLE) {
+		g_print ("%s %d,%d\n", html_type_name (HTML_OBJECT_TYPE (obj)),
+			 HTML_TABLE (obj)->totalRows, HTML_TABLE (obj)->totalCols);
 	} else
 		g_print ("%s\n", html_type_name (HTML_OBJECT_TYPE (obj)));
 }
@@ -255,7 +278,9 @@ gtk_html_debug_dump_tree_simple (HTMLObject *o,
 		case HTML_TYPE_TABLECELL:
 			gtk_html_debug_dump_tree_simple (HTML_CLUE (obj)->head, level + 1);
 			break;
-
+		case HTML_TYPE_TABLE:
+			gtk_html_debug_dump_table_simple (obj, level + 1);
+			break;
 		default:
 			break;
 		}
