@@ -31,7 +31,7 @@
 #define HC_CLASS(x) (HTML_CLUE_CLASS (HTML_OBJECT (x)->klass))
 
 HTMLClueClass html_clue_class;
-HTMLObjectClass *parent_class = NULL;
+static HTMLObjectClass *parent_class = NULL;
 
 static void set_parent (HTMLObject *o, HTMLObject *tail, HTMLObject *parent);
 
@@ -54,6 +54,20 @@ destroy (HTMLObject *o)
 	HTML_OBJECT_CLASS (parent_class)->destroy (o);
 }
 
+static guint
+get_recursive_length (HTMLObject *self)
+{
+	HTMLObject *o = HTML_CLUE (self)->head;
+	guint len = 0;
+
+	while (o) {
+		len += html_object_get_recursive_length (o);
+		o = o->next;
+	}
+
+	return len;
+}
+
 static void
 copy (HTMLObject *self,
       HTMLObject *dest)
@@ -87,7 +101,8 @@ op_helper (HTMLObject *self, HTMLEngine *e, GList *from, GList *to, GList *left,
 		html_clue_append (HTML_CLUE (cc), cut
 				  ? html_object_op_cut (o, e,
 							html_object_get_bound_list (o, from),
-							html_object_get_bound_list (o, to), left->next, right->next, len)
+							html_object_get_bound_list (o, to),
+							left ? left->next : NULL, right ? right->next : NULL, len)
 				  : html_object_op_copy (o, e,
 							 html_object_get_bound_list (o, from),
 							 html_object_get_bound_list (o, to), len));
@@ -125,7 +140,7 @@ op_cut (HTMLObject *self, HTMLEngine *e, GList *from, GList *to, GList *left, GL
 }
 
 static gboolean
-merge (HTMLObject *self, HTMLObject *with, HTMLEngine *e)
+merge (HTMLObject *self, HTMLObject *with, HTMLEngine *e, GList *left, GList *right)
 {
 	HTMLClue   *clue1, *clue2;
 
@@ -136,6 +151,8 @@ merge (HTMLObject *self, HTMLObject *with, HTMLEngine *e)
 	clue2->head = NULL;
 	clue2->tail = NULL;
 
+	printf ("merge clues\n");
+
 	html_object_change_set (self, HTML_CHANGE_ALL);
 	return TRUE;
 }
@@ -165,7 +182,7 @@ split (HTMLObject *self, HTMLEngine *e, HTMLObject *child, gint offset, gint lev
 	HTML_CLUE (dup)->head  = child;
 	set_parent (child, NULL, dup);
 
-	if (self->parent)
+	if (self->parent && HTML_OBJECT_TYPE (self->parent) != HTML_TYPE_TABLE)
 		html_clue_append_after (HTML_CLUE (self->parent), dup, self);
 
 	*left  = g_list_prepend (*left, self);
@@ -598,6 +615,7 @@ html_clue_class_init (HTMLClueClass *klass,
 	object_class->append_selection_string = append_selection_string;
 	object_class->head = head;
 	object_class->tail = tail;
+	object_class->get_recursive_length = get_recursive_length;
 
 	/* HTMLClue methods.  */
 	klass->get_left_clear = get_left_clear;
