@@ -41,6 +41,8 @@
 #include "htmlengine-save.h"
 #include "htmllinktext.h"
 #include "htmlpainter.h"
+#include "htmlplainpainter.h"
+#include "htmltable.h"
 #include "htmltext.h"
 #include "htmltextslave.h"	/* FIXME */
 #include "htmlsearch.h"
@@ -51,6 +53,7 @@ HTMLClueFlowClass html_clueflow_class;
 static HTMLClueClass *parent_class = NULL;
 
 #define HCF_CLASS(x) HTML_CLUEFLOW_CLASS (HTML_OBJECT (x)->klass)
+#define HTML_IS_PLAIN_PAINTER(obj)              (GTK_CHECK_TYPE ((obj), HTML_TYPE_PLAIN_PAINTER))
 
 
 static void
@@ -63,6 +66,12 @@ copy (HTMLObject *self,
 	HTML_CLUEFLOW (dest)->level = HTML_CLUEFLOW (self)->level;
 }
 
+static guint
+get_recursive_length (HTMLObject *self)
+{
+	return (*HTML_OBJECT_CLASS (parent_class)->get_recursive_length) (self) + (self->next ? 1 : 0);
+}
+
 static HTMLObject *
 op_copy (HTMLObject *self, HTMLEngine *e, GList *from, GList *to, guint *len)
 {
@@ -89,7 +98,7 @@ split (HTMLObject *self, HTMLEngine *e, HTMLObject *child, gint offset, gint lev
 }
 
 static gboolean
-merge (HTMLObject *self, HTMLObject *with, HTMLEngine *e)
+merge (HTMLObject *self, HTMLObject *with, HTMLEngine *e, GList *left, GList *right)
 {
 	HTMLClueFlow *cf1, *cf2;
 
@@ -99,7 +108,16 @@ merge (HTMLObject *self, HTMLObject *with, HTMLEngine *e)
 	html_clueflow_remove_text_slaves (cf1);
 	html_clueflow_remove_text_slaves (cf2);
 
-	return (* HTML_OBJECT_CLASS (parent_class)->merge) (self, with, e);
+	printf ("merge flows\n");
+
+	if ((HTML_IS_TABLE (HTML_CLUE (self)->tail) && !html_clueflow_is_empty (cf2))
+	    || (HTML_IS_TABLE (HTML_CLUE (with)->head) && !html_clueflow_is_empty (cf1)))
+		return FALSE;
+
+	if (html_clueflow_is_empty (cf1))
+		cf1->style = cf2->style;
+
+	return (* HTML_OBJECT_CLASS (parent_class)->merge) (self, with, e, left, right);
 }
 
 static guint
@@ -108,9 +126,14 @@ calc_padding (HTMLPainter *painter)
 	guint ascent, descent;
 
 	/* FIXME maybe this should depend on the style.  */
-	ascent = html_painter_calc_ascent (painter, GTK_HTML_FONT_STYLE_SIZE_3, NULL);
-	descent = html_painter_calc_descent (painter, GTK_HTML_FONT_STYLE_SIZE_3, NULL);
-
+	/* FIXME move this into the painter.             */
+	if (HTML_IS_PLAIN_PAINTER (painter)) {
+		ascent = 0;
+		descent = 0;
+	} else {
+		ascent = html_painter_calc_ascent (painter, GTK_HTML_FONT_STYLE_SIZE_3, NULL);
+		descent = html_painter_calc_descent (painter, GTK_HTML_FONT_STYLE_SIZE_3, NULL);
+	}
 	return ascent + descent;
 }
 
@@ -333,6 +356,20 @@ set_line_x (HTMLObject **obj, HTMLObject *run, gint x, gboolean *changed)
 	return x;
 }
 
+static gint
+pref_right_margin (HTMLPainter *p, HTMLClueFlow *clueflow, HTMLObject *o, gint y) 
+{
+	gint fixed_margin = html_object_get_right_margin (o, y);
+
+	/* FIXME: this hack lets us wrap the display at 72 characters when we are using
+	   a plain painter */
+	  
+	if (clueflow->style == HTML_CLUEFLOW_STYLE_PRE || ! HTML_IS_PLAIN_PAINTER(p))
+		return fixed_margin;
+	
+	return MIN (fixed_margin, 72 * html_painter_get_space_width (p, GTK_HTML_FONT_STYLE_SIZE_3, NULL));
+}
+
 /* EP CHECK: should be mostly OK.  */
 /* FIXME: But it's awful.  Too big and ugly.  */
 static gboolean
@@ -380,7 +417,7 @@ calc_size (HTMLObject *o,
 	lmargin = html_object_get_left_margin (o->parent, o->y);
 	if (indent > lmargin)
 		lmargin = indent;
-	rmargin = html_object_get_right_margin (o->parent, o->y);
+	rmargin = pref_right_margin (painter, HTML_CLUEFLOW (o), o->parent, o->y);
 
 	w = lmargin;
 	a = 0;
@@ -453,7 +490,7 @@ calc_size (HTMLObject *o,
 					html_clue_append_right_aligned (HTML_CLUE (o->parent),
 									HTML_CLUE (c));
 
-					rmargin = html_object_get_right_margin (o->parent, o->y);
+					rmargin = pref_right_margin (painter, HTML_CLUEFLOW (o), o->parent, o->y);
 				}
 			}
 
@@ -595,7 +632,7 @@ calc_size (HTMLObject *o,
 				if (indent > lmargin)
 					lmargin = indent;
 
-				rmargin = html_object_get_right_margin (o->parent, o->y);
+				rmargin = pref_right_margin (painter, HTML_CLUEFLOW (o), o->parent, o->y);
 			}
 		}
 		
@@ -697,7 +734,7 @@ calc_size (HTMLObject *o,
 			lmargin = html_object_get_left_margin (o->parent, o->y);
 			if (indent > lmargin)
 				lmargin = indent;
-			rmargin = html_object_get_right_margin (o->parent, o->y);
+			rmargin = pref_right_margin (painter, HTML_CLUEFLOW (o), o->parent, o->y);
 
 			w = lmargin;
 			d = 0;
@@ -713,10 +750,6 @@ calc_size (HTMLObject *o,
 	if (o->width < o->max_width)
 		o->width = o->max_width;
 
-#if 0
-	if (o->width > rmargin - o->x)
-		o->width = rmargin - o->x;
-#endif
 	add_post_padding (HTML_CLUEFLOW (o), padding);
 
 	if (o->ascent != old_ascent || o->descent != old_descent || o->width != old_width)
@@ -729,19 +762,21 @@ static gint
 calc_preferred_width (HTMLObject *o,
 		      HTMLPainter *painter)
 {
-	HTMLObject *obj;
+	HTMLObject *obj, *next;
 	gint maxw = 0, w = 0;
 
+	next = NULL;
+
 	for (obj = HTML_CLUE (o)->head; obj != 0; obj = obj->next) {
 		if (!(obj->flags & HTML_OBJECT_FLAG_NEWLINE)) {
 			w += html_object_calc_preferred_width (obj, painter);
 		}
 
-		if (obj->flags & HTML_OBJECT_FLAG_NEWLINE || !html_object_next_not_slave (obj)) {
+		if (obj->flags & HTML_OBJECT_FLAG_NEWLINE || !(next = html_object_next_not_slave (obj))) {
 			HTMLObject *eol = (obj->flags & HTML_OBJECT_FLAG_NEWLINE) ? html_object_prev_not_slave (obj) : obj;
 
-			/* remove trailing space width on the end of line */
-			if (html_object_is_text (eol))
+			/* remove trailing space width on the end of line which is not on end of paragraph */
+			if (next && html_object_is_text (eol))
 				w -= html_text_trail_space_width (HTML_TEXT (eol), painter);
 
 			if (w > maxw)
@@ -884,7 +919,7 @@ check_point (HTMLObject *self,
 	p = clue->head;
 	if (html_object_is_text (p))
 		p = p->next;
-	if (x < p->x || y < p->y - p->ascent) {
+	if (p && (x < p->x || y < p->y - p->ascent)) {
 		obj = html_object_check_point (p, painter, p->x, p->y - p->ascent, offset_return, for_cursor);
 		if (obj != NULL)
 			return obj;
@@ -892,7 +927,8 @@ check_point (HTMLObject *self,
 	for (p = clue->head; p != NULL; p = pnext) {
 		if (html_object_is_text (p))
 			p = p->next;
-
+		if (!p)
+			break;
 		pnext = p->next;
 		while (pnext && html_object_is_text (pnext))
 			pnext = pnext->next;
@@ -1010,9 +1046,11 @@ write_pre_tags (HTMLClueFlow *self,
 
 	prev = HTML_CLUEFLOW (HTML_OBJECT (self)->prev);
 	if (prev != NULL && prev->level == self->level && prev->style == self->style) {
-		if (! is_item (self))
+		if (!is_item (self) && self->style != HTML_CLUEFLOW_STYLE_PRE) {
 			return html_engine_save_output_string (state, "<br>\n");
-		else
+		} else if (self->style == HTML_CLUEFLOW_STYLE_PRE) {
+			return html_engine_save_output_string (state, "\n");
+		} else
 			return TRUE;
 	}
 
@@ -1129,15 +1167,13 @@ save (HTMLObject *self,
 	return write_post_tags (HTML_CLUEFLOW (self), state);
 }
 
-#define MATCH_UTF8_NBSP(s) (*s == (guchar)0xc2 && *(s + 1) == (guchar)0xa0)
- 
 static gint
 string_append_nonbsp (GString *out, guchar *s, gint length)
 {
 	gint len = length;
 
 	while (len--) {
-		if (MATCH_UTF8_NBSP (s)) {
+		if (IS_UTF8_NBSP (s)) {
 			g_string_append_c (out, ' ');
 			s += 2;
 			len--;
@@ -1202,7 +1238,7 @@ save_plain (HTMLObject *self,
 							      requested_width - pad)) {
 		guchar *s, *space;
 		
-		if (get_pre_padding (flow, 1) > 0)
+		if (get_pre_padding (flow, calc_padding (state->engine->painter)) > 0)
 			g_string_append (out, "\n");
 
 		s = html_engine_save_buffer_peek_text (buffer_state);
@@ -1220,8 +1256,8 @@ save_plain (HTMLObject *self,
 					space = s + unicode_offset_to_index (s, requested_width - pad);
 					while (space > s 
 					       && (*space != ' '))
-						// || (MATCH_UTF8_NBSP ((guchar *)unicode_next_utf8 (space)))
-						// || (MATCH_UTF8_NBSP ((guchar *)unicode_previous_utf8 (s, space)))))
+						// || (IS_UTF8_NBSP ((guchar *)unicode_next_utf8 (space)))
+						// || (IS_UTF8_NBSP ((guchar *)unicode_previous_utf8 (s, space)))))
 						space = unicode_previous_utf8 (s, space);
 					
 					if (space != s)
@@ -1233,7 +1269,7 @@ save_plain (HTMLObject *self,
 			s += string_append_nonbsp (out, s, len);
 			
 			/* Trim the space at the end */
-			while (*s == ' ' || MATCH_UTF8_NBSP (s)) 
+			while (*s == ' ' || IS_UTF8_NBSP (s)) 
 				s = unicode_next_utf8 (s);
 			
 			if (*s == '\n') 
@@ -1243,7 +1279,7 @@ save_plain (HTMLObject *self,
 			firstline = FALSE;
 		}
 		
-		if (get_post_padding (flow, 1) > 0)
+		if (get_post_padding (flow, calc_padding (state->engine->painter)) > 0)
 			g_string_append (out, "\n");
 				
 	}
@@ -1576,6 +1612,7 @@ html_clueflow_class_init (HTMLClueFlowClass *klass,
 	object_class->search = search;
 	object_class->search_next = search_next;
 	object_class->relayout = relayout;
+	object_class->get_recursive_length = get_recursive_length;
 
 	klass->get_default_font_style = get_default_font_style;
 
@@ -2094,3 +2131,17 @@ html_clueflow_spell_check (HTMLClueFlow *flow, HTMLEngine *e, HTMLInterval *inte
 		html_interval_destroy (new_interval);
 	}
 }
+
+gboolean
+html_clueflow_is_empty (HTMLClueFlow *flow)
+{
+	HTMLClue *clue;
+	g_return_val_if_fail (HTML_IS_CLUEFLOW (flow), TRUE);
+
+	clue = HTML_CLUE (flow);
+
+	if (clue->head && html_object_is_text (clue->head)
+	    && HTML_TEXT (clue->head)->text_len == 0 && !html_object_next_not_slave (clue->head))
+		return TRUE;
+	return FALSE;
+}
