@@ -24,6 +24,7 @@
 #include <config.h>
 #include <string.h>
 #include <stdio.h>
+#include <gtk/gtksignal.h>
 #include "gtkhtml.h"
 #include "gtkhtml-embedded.h"
 #include "htmlembedded.h"
@@ -68,7 +69,6 @@ draw (HTMLObject *o,
 		return;
 
 	if (element->widget) {
-
 		new_x = GTK_LAYOUT (element->parent)->hadjustment->value + o->x + tx;
 		new_y = GTK_LAYOUT (element->parent)->vadjustment->value + o->y + ty - o->ascent;
 		
@@ -83,6 +83,8 @@ draw (HTMLObject *o,
 		if (!element->widget->parent)
 			gtk_layout_put (GTK_LAYOUT(element->parent), element->widget, new_x, new_y);
 	}
+
+	html_painter_draw_embedded (p, element, tx, ty);
 }
 
 static void
@@ -142,27 +144,30 @@ calc_size (HTMLObject *self,
 	GtkWidget *widget;
 	HTMLEmbedded *emb = HTML_EMBEDDED (self);
 	gint pixel_size;
-	gint old_width, old_ascent;
+	gint old_width, old_ascent, old_descent;
 	GtkRequisition requisition;
 
 	widget = emb->widget;
-	if (widget == NULL || !GTK_WIDGET_REALIZED (widget))
+	if (widget == NULL)
 		return FALSE;
 
 	pixel_size = html_painter_get_pixel_size (painter);
 
 	old_width = self->width;
 	old_ascent = self->ascent;
+	old_descent = self->descent;
 
 	gtk_widget_size_request (widget, &requisition);
+	
+	if (GTK_IS_HTML_EMBEDDED(widget))
+		self->descent = GTK_HTML_EMBEDDED (widget)->descent * pixel_size;
+	else
+		self->descent = 0;
 
 	self->width  = requisition.width  * pixel_size;
-	self->ascent = requisition.height * pixel_size;
+	self->ascent = requisition.height * pixel_size - self->descent;
 
-	/* This never changes.  */
-	self->descent = 0;
-
-	if (old_width != self->width || old_ascent != self->ascent)
+	if (old_width != self->width || old_ascent != self->ascent || old_ascent != self->descent)
 		return TRUE;
 
 	return FALSE;
@@ -237,7 +242,6 @@ html_embedded_encode_string (gchar *str)
         return ptr;
 }
 
-
 void
 html_embedded_type_init (void)
 {
@@ -298,31 +302,6 @@ html_embedded_init (HTMLEmbedded *element,
 	element->abs_x  = element->abs_y = -1;
 }
 
-void
-html_embedded_size_recalc (HTMLEmbedded *em)
-{
-	HTMLObject *o;
-	GtkHTMLEmbedded *eb;
-	GtkRequisition req;
-
-	o = HTML_OBJECT (em);
-
-	if (em->widget == NULL)
-		return;
-	eb = (GtkHTMLEmbedded *)em->widget;
-
-	gtk_widget_size_request (em->widget, &req);
-
-	o->width = req.width;
-	if (GTK_IS_HTML_EMBEDDED(eb)) {
-		o->descent = eb->descent;
-	} else {
-		o->descent = 0;
-	}
-
-	o->ascent = req.height - o->descent;
-}
-
 static gboolean
 html_embedded_grab_cursor(GtkWidget *eb, GdkEvent *event)
 {
@@ -361,11 +340,20 @@ allocate (GtkWidget *w, GtkAllocation  *allocation, HTMLEmbedded *e)
 }
 
 void
-html_embedded_set_widget (HTMLEmbedded *e, GtkWidget *w)
+html_embedded_set_widget (HTMLEmbedded *emb, GtkWidget *w)
 {
-	e->widget = w;
+	emb->widget = w;
 	
 	gtk_widget_show (w);
+
+	gtk_object_set_data (GTK_OBJECT (w), "embeddedelement", emb);
 	gtk_signal_connect (GTK_OBJECT (w), "size_allocate",
-			    GTK_SIGNAL_FUNC (allocate), e);
+			    GTK_SIGNAL_FUNC (allocate), emb);
 }
+
+GtkWidget *
+html_embedded_get_widget (HTMLEmbedded *e)
+{
+	return e->widget;
+}
+
