@@ -40,6 +40,7 @@
 #include "htmlinterval.h"
 #include "htmllinktext.h"
 #include "htmlobject.h"
+#include "htmltable.h"
 #include "htmlselection.h"
 #include "htmlsettings.h"
 #include "htmltext.h"
@@ -127,8 +128,18 @@ html_point_get_right (HTMLPoint *source, HTMLPoint *dest)
 	*dest = *source;
 }
 
+static void
+object_get_parent_list (HTMLObject *o, gint level, GList **list)
+{
+	while (level > 0 && o) {
+		*list = g_list_prepend (*list, o);
+		o = o->parent;
+		level--;
+	}
+}
+
 static GList *
-get_parent_list (HTMLPoint *point, gint level, gboolean include_offset)
+point_get_parent_list (HTMLPoint *point, gint level, gboolean include_offset)
 {
 	GList *list;
 	HTMLObject *o;
@@ -136,61 +147,92 @@ get_parent_list (HTMLPoint *point, gint level, gboolean include_offset)
 	list = include_offset ? g_list_prepend (NULL, GINT_TO_POINTER (point->offset)) : NULL;
 	o    = point->object;
 
-	while (level > 0 && o) {
-		list = g_list_prepend (list, o);
-		o = o->parent;
-		level--;
-	}
+	object_get_parent_list (point->object, level, &list);
 
 	return list;
 }
 
 static gint
-get_parent_level (HTMLObject *from, HTMLObject *to)
+get_parent_depth (HTMLObject *o, HTMLObject *parent)
 {
 	gint level = 1;
 
-	while (from && to && from != to) {
-		from = from->parent;
-		to   = to->parent;
-		level++;
+	while (o && parent && o != parent) {
+		o = o->parent;
+		level ++;
 	}
 
 	return level;
 }
 
+static gboolean
+is_parent (HTMLObject *o,HTMLObject *parent)
+{
+	while (o) {
+		if (o == parent)
+			return TRUE;
+		o = o->parent;
+	}
+
+	return FALSE;
+}
+
+static HTMLObject *
+try_find_common_parent_of (HTMLObject *child, HTMLObject *parent)
+{
+	while (parent) {
+		if (is_parent (child, parent))
+			return parent;
+		parent = parent->parent;
+	}
+
+	return NULL;
+}
+
+static HTMLObject *
+get_common_parent (HTMLObject *from, HTMLObject *to)
+{
+	HTMLObject *parent;
+
+	parent = try_find_common_parent_of (from, to);
+
+	return parent ? parent : try_find_common_parent_of (to, from);
+}
+
 static void
 prepare_delete_bounds (HTMLEngine *e, GList **from_list, GList **to_list,
 		       GList **bound_left, GList **bound_right)
 {
 	HTMLPoint b_left, b_right, begin, end;
-	gint level;
+	HTMLObject *common_parent;
 
 	g_assert (e->selection);
 
 	html_point_get_right (&e->selection->from, &begin);
 	html_point_get_left  (&e->selection->to,   &end);
 
-	level = get_parent_level (begin.object, end.object);
+	common_parent = get_common_parent (begin.object, end.object);
 
-	*from_list = get_parent_list (&begin, level, TRUE);
-	*to_list   = get_parent_list (&end,   level, TRUE);
+	*from_list = point_get_parent_list (&begin, get_parent_depth (begin.object, common_parent), TRUE);
+	*to_list   = point_get_parent_list (&end,   get_parent_depth (end.object, common_parent),   TRUE);
 
 	if (bound_left && bound_right) {
+		gint level;
+
 		html_point_get_left  (&e->selection->from, &b_left);
 		html_point_get_right (&e->selection->to,   &b_right);
 
-		level = get_parent_level (b_left.object, b_right.object);
+		common_parent = get_common_parent (b_left.object, b_right.object);
 
-		*bound_left  = b_left.object ? get_parent_list (&b_left, level - 1, FALSE) : NULL;
-		*bound_right = b_right.object ? get_parent_list (&b_right, level - 1, FALSE) : NULL;
+		level = get_parent_depth (b_left.object, common_parent);
+		*bound_left  = b_left.object  ? point_get_parent_list (&b_left, level - 1, FALSE) : NULL;
+		if (level > 1 && *bound_left)
+			*bound_left  = g_list_prepend (*bound_left, NULL);
 
-		if (level > 1) {
-			if (*bound_left)
-				*bound_left  = g_list_prepend (*bound_left, NULL);
-			if (*bound_right)
-				*bound_right = g_list_prepend (*bound_right, NULL);
-		}
+		level = get_parent_depth (b_right.object, common_parent);
+		*bound_right = b_right.object ? point_get_parent_list (&b_right, level - 1, FALSE) : NULL;
+		if (level > 1 && *bound_right)
+			*bound_right = g_list_prepend (*bound_right, NULL);
 	}
 }
 
@@ -218,6 +260,9 @@ remove_empty_and_merge (HTMLEngine *e, gboolean merge, GList *left, GList *right
 		ro  = HTML_OBJECT (right->data);
 		len = html_object_get_length (lo);
 
+		left  = left ? left->next : NULL;
+		right = right ? right->next : NULL;
+
 		if (html_object_is_text (lo) && !*HTML_TEXT (lo)->text && (html_object_prev_not_slave (lo) || merge)) {
 			HTMLObject *nlo = html_object_prev_not_slave (lo);
 
@@ -241,15 +286,13 @@ remove_empty_and_merge (HTMLEngine *e, gboolean merge, GList *left, GList *right
 		}
 
 		if (merge && lo && ro) {
-			if (!html_object_merge (lo, ro, e))
+			if (!html_object_merge (lo, ro, e, left, right))
 				break;
 			if (ro == e->cursor->object) {
 				e->cursor->object  = lo;
 				e->cursor->offset += len;
 			}
 		}
-		left  = left->next;
-		right = right->next;
 	}
 
 	prev = html_object_prev_not_slave (e->cursor->object);
@@ -258,14 +301,97 @@ remove_empty_and_merge (HTMLEngine *e, gboolean merge, GList *left, GList *right
 		e->cursor->offset = html_object_get_length (e->cursor->object);
 	}
 	/* printf ("-- after\n");
-	   gtk_html_debug_dump_tree_simple (e->clue, 0);
-	   printf ("-- END merge\n"); */
+	gtk_html_debug_dump_tree_simple (e->clue, 0);
+	printf ("-- END merge\n"); */
+}
+
+static gboolean
+look_for_non_appendable (GList *list)
+{
+	while (list) {
+		HTMLObject *o = HTML_OBJECT (list->data);
+
+		if (o->parent && HTML_OBJECT_TYPE (o->parent) == HTML_TYPE_TABLE)
+			return TRUE;
+		list = list->next;
+	}
+
+	return FALSE;
+}
+
+static HTMLObject *
+split_between_objects (HTMLObject *object, HTMLObject *other,
+		       GList **object_parents, GList **other_parents, gint *object_level)
+{
+	HTMLObject *common_parent;
+	gint d_object, d_other;
+
+	common_parent = get_common_parent (object, other);
+
+	if (common_parent) {
+		d_object = get_parent_depth (object, common_parent);
+		d_other  = get_parent_depth (other,  common_parent);
+
+		if (d_object <= *object_level) {
+			object_get_parent_list (object, d_object - 1, object_parents);
+			object_get_parent_list (other, d_other - 1, other_parents);
+
+			if (*object_parents) {
+				printf ("split object\n");
+				gtk_html_debug_dump_tree_simple ((*object_parents)->data, 0);
+			}
+
+			if (*other_parents) {
+				printf ("split other\n");
+				gtk_html_debug_dump_tree_simple ((*other_parents)->data, 0);
+			}
+
+			if (!look_for_non_appendable (*object_parents) && !look_for_non_appendable (*object_parents)) {
+				g_list_free (*object_parents);
+				g_list_free (*other_parents);
+
+				printf ("drop lists, do simple split\n");
+				*object_parents = NULL;
+				*other_parents = NULL;
+
+				return NULL;
+			}
+
+			*object_level -= d_object - 1;
+
+			return common_parent;
+		}
+	}
+
+	return NULL;
 }
 
 static void
 split_and_add_empty_texts (HTMLEngine *e, gint level, GList **left, GList **right)
 {
-	html_object_split (e->cursor->object, e, NULL, e->cursor->offset, level, left, right);
+	HTMLObject *object = NULL;
+
+	if ((e->cursor->offset == 0 || (html_object_get_length (e->cursor->object) == e->cursor->offset))) {
+		HTMLObject *leaf;
+		gboolean prev;
+
+		prev = html_object_get_length (e->cursor->object) != e->cursor->offset;
+
+		leaf = prev
+		? html_object_prev_leaf_not_type (e->cursor->object, HTML_TYPE_TEXTSLAVE)
+		: html_object_next_leaf_not_type (e->cursor->object, HTML_TYPE_TEXTSLAVE);
+		if (leaf) {
+			if (prev)
+				object = split_between_objects (e->cursor->object, leaf, right, left, &level);
+			else
+				object = split_between_objects (e->cursor->object, leaf, left, right, &level);
+		}
+	}
+
+	if (!object)
+		object = e->cursor->object;
+
+	html_object_split (object, e, *right ? HTML_OBJECT ((*right)->data) : NULL, e->cursor->offset, level, left, right);
 }
 
 /* end of helper */
@@ -281,6 +407,8 @@ html_engine_copy (HTMLEngine *e)
 		e->clipboard_len = 0;
 		e->clipboard     = html_object_op_copy (HTML_OBJECT (from->data), e, from->next, to->next,
 							&e->clipboard_len);
+		printf ("copy len: %d (parent %p)\n", e->clipboard_len, e->clipboard->parent);
+		gtk_html_debug_dump_tree_simple (e->clipboard, 0);
 		html_engine_thaw (e);
 	}
 }
@@ -306,10 +434,12 @@ static void
 delete_undo_action (HTMLEngine *e, HTMLUndoData *data, HTMLUndoDirection dir)
 {
 	DeleteUndo *undo;
+	HTMLObject *buffer;
+	guint       len = 0;
 
-	undo = (DeleteUndo *) data;
-	insert_object (e, undo->buffer, undo->buffer_len, html_undo_direction_reverse (dir), TRUE);
-	undo->buffer = NULL;
+	undo         = (DeleteUndo *) data;
+	buffer       = html_object_op_copy (undo->buffer, e, NULL, NULL, &len);
+	insert_object (e, buffer, undo->buffer_len, html_undo_direction_reverse (dir), TRUE);
 }
 
 static void
@@ -324,6 +454,8 @@ delete_setup_undo (HTMLEngine *e, HTMLObject *buffer, guint len, HTMLUndoDirecti
 	undo->buffer       = buffer;
 	undo->buffer_len   = len;
 
+	/* printf ("delete undo len %d\n", len); */
+
 	html_undo_add_action (e->undo,
 			      html_undo_action_new ("Delete", delete_undo_action,
 						    HTML_UNDO_DATA (undo), html_cursor_get_position (e->cursor)),
@@ -415,6 +547,8 @@ html_engine_cut (HTMLEngine *e)
 {
 	html_engine_clipboard_clear (e);
 	delete_object (e, &e->clipboard, &e->clipboard_len, HTML_UNDO_UNDO);
+	printf ("cut  len: %d\n", e->clipboard_len);
+	gtk_html_debug_dump_tree_simple (e->clipboard, 0);
 }
 
 /*
@@ -430,17 +564,30 @@ insert_object_do (HTMLEngine *e, HTMLObject *obj, guint len, gboolean check)
 	GList *first = NULL, *last = NULL;
 	gint level;
 
-	html_engine_freeze (e);
-	orig = html_cursor_dup (e->cursor);
-
 	/* FIXME for tables */
+	if (obj->klass->type == HTML_TYPE_TABLE) {
+		html_engine_append_object (e, obj, len);
+		return;
+	}
+
+	html_engine_freeze (e);
+	if (HTML_IS_TABLE (e->cursor->object)) {
+		gint offset = e->cursor->offset;
+
+		html_engine_insert_empty_paragraph (e);
+		if (offset == 0)
+			html_cursor_backward (e->cursor, e);
+	}
+
 	level = 0;
 	cur   = html_object_get_head_leaf (obj);
 	while (cur) {
 		level++;
 		cur = cur->parent;
 	}
+	orig = html_cursor_dup (e->cursor);
 
+	html_object_change_set_down (obj, HTML_CHANGE_ALL);
 	split_and_add_empty_texts (e, level, &left, &right);
         get_tree_bounds_for_merge (obj, &first, &last);
 
@@ -499,6 +646,8 @@ insert_setup_undo (HTMLEngine *e, guint len, HTMLUndoDirection dir)
 	html_undo_data_init (HTML_UNDO_DATA (undo));
 	undo->len = len;
 
+	/* printf ("insert undo len %d\n", len); */
+
 	html_undo_add_action (e->undo,
 			      html_undo_action_new ("Insert", insert_undo_action,
 						    HTML_UNDO_DATA (undo), html_cursor_get_position (e->cursor)),
@@ -521,7 +670,7 @@ html_engine_insert_object (HTMLEngine *e, HTMLObject *o, guint len)
 void
 html_engine_paste_object (HTMLEngine *e, HTMLObject *o, guint len)
 {
-	html_undo_level_begin (e->undo, "Paste");
+	html_undo_level_begin (e->undo, "Paste", "Paste");
 	html_engine_delete (e);
 	html_engine_insert_object (e, o, len);
 	html_undo_level_end (e->undo);
@@ -559,6 +708,7 @@ html_engine_insert_empty_paragraph (HTMLEngine *e)
 	split_and_add_empty_texts (e, 2, &left, &right);
 	remove_empty_and_merge (e, FALSE, left, right, orig);
 	html_cursor_forward (e->cursor, e);
+
 	insert_setup_undo (e, 1, HTML_UNDO_UNDO);
 	g_list_free (left);
 	g_list_free (right);
@@ -572,6 +722,9 @@ html_engine_insert_empty_paragraph (HTMLEngine *e)
 	html_engine_thaw (e);
 
 	gtk_html_editor_event_command (e->widget, GTK_HTML_COMMAND_INSERT_PARAGRAPH);
+
+	/* break links in new paragraph */
+	html_engine_insert_link (e, NULL, NULL);
 }
 
 void
@@ -581,7 +734,7 @@ html_engine_insert_text (HTMLEngine *e, const gchar *text, guint len)
 	gint alen;
 
 	if (len == -1)
-		len = strlen (text);
+		len = unicode_strlen (text, -1);
 	if (!len)
 		return;
 
@@ -593,6 +746,11 @@ html_engine_insert_text (HTMLEngine *e, const gchar *text, guint len)
 			gboolean check = FALSE;
 
 			check_magic_link (e, text, alen);
+
+			/* stop inserting links after space */
+			if (*text == ' ')
+				html_engine_set_insertion_link (e, NULL, NULL);
+
 			o = html_engine_new_text (e, text, alen);
 			html_text_convert_nbsp (HTML_TEXT (o), TRUE);
 
@@ -614,7 +772,12 @@ html_engine_insert_text (HTMLEngine *e, const gchar *text, guint len)
 void
 html_engine_paste_text (HTMLEngine *e, const gchar *text, guint len)
 {
-	html_undo_level_begin (e->undo, "Paste text");
+	gchar *undo_name = g_strdup_printf ("Paste text: '%s'", text);
+	gchar *redo_name = g_strdup_printf ("Unpaste text: '%s'", text);
+
+	html_undo_level_begin (e->undo, undo_name, redo_name);
+	g_free (undo_name);
+	g_free (redo_name);
 	html_engine_delete (e);
 	html_engine_insert_text (e, text, len);
 	html_undo_level_end (e->undo);
@@ -677,7 +840,7 @@ change_link (HTMLObject *o, HTMLEngine *e, gpointer data)
 			html_clue_remove (HTML_CLUE (o->parent), o);
 			html_object_destroy (o);
 			if (changed->prev)
-				html_object_merge (changed->prev, changed, e);
+				html_object_merge (changed->prev, changed, e, NULL, NULL);
 		} else {
 			html_object_destroy (e->clipboard);
 			e->clipboard     = changed;
@@ -686,6 +849,15 @@ change_link (HTMLObject *o, HTMLEngine *e, gpointer data)
 	}
 }
 
+void
+html_engine_set_insertion_link (HTMLEngine *e, const gchar *url, const gchar *target)
+{
+	html_engine_set_url    (e, url);
+	html_engine_set_target (e, target);
+	if (!url && e->insertion_color == html_colorset_get_color (e->settings->color_set, HTMLLinkColor))
+		html_engine_set_color (e, html_colorset_get_color (e->settings->color_set, HTMLTextColor));
+}
+
 void
 html_engine_insert_link (HTMLEngine *e, const gchar *url, const gchar *target)
 {
@@ -697,9 +869,45 @@ html_engine_insert_link (HTMLEngine *e, const gchar *url, const gchar *target)
 		data.color  = url
 			? html_colorset_get_color (e->settings->color_set, HTMLLinkColor)
 			: html_colorset_get_color (e->settings->color_set, HTMLTextColor);
-		html_engine_cut_and_paste (e, url ? "Insert link" : "Remove link", change_link, &data);
+		html_engine_cut_and_paste (e,
+					   url ? "Insert link" : "Remove link",
+					   url ? "Remove link" : "Insert link",
+					   change_link, &data);
+	} else
+		html_engine_set_insertion_link (e, url, target);
+}
+
+void
+html_engine_append_object (HTMLEngine *e, HTMLObject *o, guint len)
+{
+	GList *left = NULL, *right = NULL;
+	HTMLObject *where;
+
+	html_engine_freeze (e);
+	if (html_clueflow_is_empty (HTML_CLUEFLOW (e->cursor->object->parent))) {
+		HTMLObject *c, *cn;
+		HTMLClue *clue = HTML_CLUE (e->cursor->object->parent);
+		for (c = clue->head; c; c = cn) {
+			cn = c->next;
+			html_object_destroy (c);
+		}
+		clue->head = clue->tail = o;
+		e->cursor->object = o;
+		e->cursor->offset = 0;
+		o->parent = HTML_OBJECT (clue);
 	} else {
-		html_engine_set_url    (e, url);
-		html_engine_set_target (e, target);
+		HTMLObject *flow;
+
+		flow  = html_clueflow_new (HTML_CLUEFLOW_STYLE_NORMAL, 0);
+		html_clue_append (HTML_CLUE (flow), o);
+
+		html_object_split (e->cursor->object, e, NULL, e->cursor->offset, 2, &left, &right);
+
+		where = HTML_OBJECT (left->data);
+		html_clue_append_after (HTML_CLUE (where->parent), flow, where);
 	}
+
+	html_cursor_forward_n (e->cursor, e, len);
+	insert_setup_undo (e, len, HTML_UNDO_UNDO);	
+	html_engine_thaw (e);
 }
