@@ -42,7 +42,7 @@ get_font_style_from_selection (HTMLEngine *engine)
 	GtkHTMLFontStyle style;
 	GtkHTMLFontStyle conflicts;
 	gboolean first;
-	HTMLObject *p;
+	HTMLPoint p;
 
 	g_return_val_if_fail (engine->clue != NULL, GTK_HTML_FONT_STYLE_DEFAULT);
 	g_assert (html_engine_is_selection_active (engine));
@@ -57,22 +57,22 @@ get_font_style_from_selection (HTMLEngine *engine)
 	conflicts = GTK_HTML_FONT_STYLE_DEFAULT;
 	first = TRUE;
 
-	p = engine->selection->from.object;
+	p = engine->selection->from;
 
 	while (1) {
-		if (html_object_is_text (p)) {
+		if (html_object_is_text (p.object) && p.offset != html_object_get_length (p.object)) {
 			if (first) {
-				style = HTML_TEXT (p)->font_style;
+				style = HTML_TEXT (p.object)->font_style;
 				first = FALSE;
 			} else
-				conflicts |= HTML_TEXT (p)->font_style ^ style;
+				conflicts |= HTML_TEXT (p.object)->font_style ^ style;
 		}
 
-		if (p == engine->selection->to.object)
+		if (html_point_cursor_object_eq (&p, &engine->selection->to))
 			break;
 
-		p = html_object_next_leaf (p);
-		g_assert (p != NULL);
+		html_point_next_cursor (&p);
+		g_assert (p.object != NULL);
 	}
 
 	return style & ~conflicts;
@@ -82,22 +82,22 @@ static HTMLColor *
 get_color_from_selection (HTMLEngine *engine)
 {
 	HTMLColor *color = NULL;
-	HTMLObject *p;
+	HTMLPoint p;
 
 	g_return_val_if_fail (engine->clue != NULL, NULL);
 	g_assert (html_engine_is_selection_active (engine));
 
-	p = engine->selection->from.object;
+	p = engine->selection->from;
 	while (1) {
-		if (html_object_is_text (p)) {
-			color = HTML_TEXT (p)->color;
+		if (html_object_is_text (p.object)  && p.offset != html_object_get_length (p.object)) {
+			color = HTML_TEXT (p.object)->color;
 			break;
 		}
 
-		if (p == engine->selection->to.object)
+		if (html_point_cursor_object_eq (&p, &engine->selection->to))
 			break;
-		p = html_object_next_leaf (p);
-		g_assert (p != NULL);
+		html_point_next_cursor (&p);
+		g_assert (p.object != NULL);
 	}
 
 	return color;
@@ -106,43 +106,43 @@ get_color_from_selection (HTMLEngine *engine)
 GtkHTMLFontStyle
 html_engine_get_document_font_style (HTMLEngine *engine)
 {
-	HTMLObject *curr;
-
 	g_return_val_if_fail (engine != NULL, GTK_HTML_FONT_STYLE_DEFAULT);
 	g_return_val_if_fail (HTML_IS_ENGINE (engine), GTK_HTML_FONT_STYLE_DEFAULT);
 	g_return_val_if_fail (engine->editable, GTK_HTML_FONT_STYLE_DEFAULT);
 
-	curr = engine->cursor->object;
-
-	if (curr == NULL)
-		return GTK_HTML_FONT_STYLE_DEFAULT;
-	else if (! html_object_is_text (curr))
-		return GTK_HTML_FONT_STYLE_DEFAULT;
-	else if (! html_engine_is_selection_active (engine))
-		return HTML_TEXT (curr)->font_style;
-	else
+	if (html_engine_is_selection_active (engine))
 		return get_font_style_from_selection (engine);
+	else {
+		HTMLObject *curr = engine->cursor->object;
+
+		if (curr == NULL)
+			return GTK_HTML_FONT_STYLE_DEFAULT;
+		else if (! html_object_is_text (curr))
+			return GTK_HTML_FONT_STYLE_DEFAULT;
+		else
+			return HTML_TEXT (curr)->font_style;
+	}
 }
 
 HTMLColor *
 html_engine_get_document_color (HTMLEngine *engine)
 {
-	HTMLObject *curr;
-
 	g_return_val_if_fail (engine != NULL, NULL);
 	g_return_val_if_fail (HTML_IS_ENGINE (engine), NULL);
 	g_return_val_if_fail (engine->editable, NULL);
 
-	curr = engine->cursor->object;
-
-	if (curr == NULL)
-		return NULL;
-	else if (! html_object_is_text (curr))
-		return NULL;
-	else if (! html_engine_is_selection_active (engine))
-		return HTML_TEXT (curr)->color;
-	else
+	if (html_engine_is_selection_active (engine))
 		return get_color_from_selection (engine);
+	else {
+		HTMLObject *curr = engine->cursor->object;
+
+		if (curr == NULL)
+			return NULL;
+		else if (! html_object_is_text (curr))
+			return NULL;
+		else
+			return HTML_TEXT (curr)->color;
+	}
 }
 
 GtkHTMLFontStyle
@@ -237,11 +237,11 @@ object_set_font_style (HTMLObject *o, HTMLEngine *e, gpointer data)
 	if (html_object_is_text (o)) {
 		struct tmp_font *tf = (struct tmp_font *) data;
 
-		HTML_TEXT (o)->font_style |= tf->or_mask;
 		HTML_TEXT (o)->font_style &= tf->and_mask;
+		HTML_TEXT (o)->font_style |= tf->or_mask;
 
 		if (o->prev)
-			html_object_merge (o->prev, o, e);
+			html_object_merge (o->prev, o, e, NULL, NULL);
 	}
 }
 
@@ -254,18 +254,20 @@ html_engine_set_font_style (HTMLEngine *e,
 	g_return_val_if_fail (HTML_IS_ENGINE (e), FALSE);
 	g_return_val_if_fail (e->editable, FALSE);
 
+
+	/* printf ("and %d or %d\n", and_mask, or_mask); */
 	if (html_engine_is_selection_active (e)) {
 		struct tmp_font *tf = g_new (struct tmp_font, 1);
 		tf->and_mask = and_mask;
 		tf->or_mask  = or_mask;
-		html_engine_cut_and_paste (e, "Set font style", object_set_font_style, tf);
+		html_engine_cut_and_paste (e, "Set font style", "Unset font style", object_set_font_style, tf);
 		g_free (tf);
 		return TRUE;
 	} else {
 		GtkHTMLFontStyle old = e->insertion_font_style;
 
-		e->insertion_font_style |= or_mask;
 		e->insertion_font_style &= and_mask;
+		e->insertion_font_style |= or_mask;
 
 		return (old == e->insertion_font_style) ? FALSE : TRUE;
 	}
@@ -310,7 +312,7 @@ inc_dec_size_cb (HTMLObject *o, HTMLEngine *e, gpointer data)
 	if (html_object_is_text (o)) {
 		html_text_set_font_style (HTML_TEXT (o), e, inc_dec_size (HTML_TEXT (o)->font_style, GPOINTER_TO_INT (data)));
 		if (o->prev)
-			html_object_merge (o->prev, o, e);
+			html_object_merge (o->prev, o, e, NULL, NULL);
 	}
 }
 
@@ -318,7 +320,9 @@ void
 html_engine_font_size_inc_dec (HTMLEngine *e, gboolean inc)
 {
 	if (html_engine_is_selection_active (e))
-		html_engine_cut_and_paste (e, (inc) ? "Increase font size" : "Decrease font size",
+		html_engine_cut_and_paste (e,
+					   inc ? "Increase font size" : "Decrease font size",
+					   inc ? "Decrease font size" : "Increase font size",
 					   inc_dec_size_cb, GINT_TO_POINTER (inc));
 	else
 		e->insertion_font_style = inc_dec_size (e->insertion_font_style, inc);
@@ -331,7 +335,7 @@ set_color (HTMLObject *o, HTMLEngine *e, gpointer data)
 		html_text_set_color (HTML_TEXT (o), NULL, (HTMLColor *) data);
 
 		if (o->prev)
-			html_object_merge (o->prev, o, e);
+			html_object_merge (o->prev, o, e, NULL, NULL);
 	}
 }
 
@@ -344,7 +348,7 @@ html_engine_set_color (HTMLEngine *e, HTMLColor *color)
 		color = html_colorset_get_color (e->settings->color_set, HTMLTextColor);
 
 	if (html_engine_is_selection_active (e)) {
-		html_engine_cut_and_paste (e, "Set color", set_color, color);
+		html_engine_cut_and_paste (e, "Set color", "Unset color", set_color, color);
 		return TRUE;
 	} else {
 
@@ -398,19 +402,19 @@ static const gchar *
 get_url_or_target_from_selection (HTMLEngine *e, gboolean get_url)
 {
 	const gchar *str = NULL;
-	HTMLObject *p;
+	HTMLPoint p;
 
 	g_return_val_if_fail (e->clue != NULL, NULL);
 	g_assert (html_engine_is_selection_active (e));
 	g_assert (e->mark != NULL);
 
-	p = e->selection->from.object;
+	p = e->selection->from;
 	while (1) {
-		str = get_url ? html_object_get_url (p) : html_object_get_target (p);
-		if (str || p == e->selection->to.object)
+		str = get_url ? html_object_get_url (p.object) : html_object_get_target (p.object);
+		if (str || html_point_cursor_object_eq (&p, &e->selection->to))
 			break;
-		p = html_object_next_leaf (p);
-		g_assert (p != NULL);
+		html_point_next_cursor (&p);
+		g_assert (p.object != NULL);
 	}
 
 	return str;
