@@ -25,8 +25,9 @@
 #include "htmlengine-edit-text.h"
 #include "htmlengine-edit-movement.h"
 #include "htmlengine-edit.h"
-#include "htmltext.h"
 #include "htmlselection.h"
+#include "htmltext.h"
+#include "htmlundo.h"
 
 static gboolean
 find_first (HTMLEngine *e)
@@ -64,14 +65,18 @@ void
 html_engine_capitalize_word (HTMLEngine *e)
 {
 	if (find_first (e)) {
+		html_undo_level_begin (e->undo, "Capitalize word", "Revert word capitalize");
 		html_engine_set_mark (e);
 		html_cursor_forward (e->cursor, e);
-		html_engine_cut_and_paste (e, "Capitalize word - up 1st", upper_lower, GINT_TO_POINTER (TRUE));
+		html_engine_cut_and_paste (e, "up 1st", "revert up 1st",
+					   upper_lower, GINT_TO_POINTER (TRUE));
 		html_engine_disable_selection (e);
 		html_engine_set_mark (e);
 		html_engine_forward_word (e);
-		html_engine_cut_and_paste (e, "Capitalize word - down rest", upper_lower, GINT_TO_POINTER (FALSE));
+		html_engine_cut_and_paste (e, "down rest", "revert down rest",
+					   upper_lower, GINT_TO_POINTER (FALSE));
 		html_engine_disable_selection (e);
+		html_undo_level_end (e->undo);
 	}
 }
 
@@ -79,10 +84,11 @@ void
 html_engine_upcase_downcase_word (HTMLEngine *e, gboolean up)
 {
 	if (find_first (e)) {
-		printf ("upcase_downcase\n");
 		html_engine_set_mark (e);
 		html_engine_forward_word (e);
-		html_engine_cut_and_paste (e, (up) ? "Upcase word" : "Downcase word",
+		html_engine_cut_and_paste (e,
+					   up ? "Upcase word" : "Downcase word",
+					   up ? "Revert word upcase" : "Revert word downcase",
 					   upper_lower, GINT_TO_POINTER (up));
 		html_engine_disable_selection (e);
 	}
