@@ -156,12 +156,12 @@ html_engine_selection_pop (HTMLEngine *e)
 }
 
 void
-html_engine_cut_and_paste_begin (HTMLEngine *e, gchar *op_name)
+html_engine_cut_and_paste_begin (HTMLEngine *e, const gchar *undo_op_name, const gchar *redo_op_name)
 {
 	html_engine_hide_cursor (e);
 	html_engine_selection_push (e);
 	html_engine_clipboard_push (e);
-	html_undo_level_begin (e->undo, op_name);
+	html_undo_level_begin (e->undo, undo_op_name, redo_op_name);
 	html_engine_cut (e);
 }
 
@@ -179,10 +179,11 @@ html_engine_cut_and_paste_end (HTMLEngine *e)
 }
 
 void
-html_engine_cut_and_paste (HTMLEngine *e, gchar *op_name, HTMLObjectForallFunc iterator, gpointer data)
+html_engine_cut_and_paste (HTMLEngine *e, const gchar *undo_op_name, const gchar *redo_op_name,
+			   HTMLObjectForallFunc iterator, gpointer data)
 {
 	html_engine_edit_selection_updater_update_now (e->selection_updater);
-	html_engine_cut_and_paste_begin (e, op_name);
+	html_engine_cut_and_paste_begin (e, undo_op_name, redo_op_name);
 	if (e->clipboard)
 		html_object_forall (e->clipboard, e, iterator, data);
 	html_engine_cut_and_paste_end (e);
@@ -401,7 +402,7 @@ html_engine_indent_pre_paragraph (HTMLEngine *e)
 	html_engine_disable_selection (e);
 	position = e->cursor->position;
 
-	html_undo_level_begin (e->undo, "Indent PRE paragraph");
+	html_undo_level_begin (e->undo, "Indent PRE paragraph", "Reverse paragraph indentation");
 	html_engine_freeze (e);
 
 	go_to_begin_of_pre_para (e);
@@ -433,3 +434,143 @@ html_engine_indent_pre_paragraph (HTMLEngine *e)
 	html_engine_thaw (e);
 	html_undo_level_end (e->undo);
 }
+
+void
+html_engine_indent_pre_line (HTMLEngine *e)
+{
+	guint position;
+	guint line_offset;
+	guint last_space;
+	HTMLObject *flow;
+	unicode_char_t uc;
+
+	g_assert (e->cursor->object);
+	if (HTML_OBJECT_TYPE (e->cursor->object->parent) != HTML_TYPE_CLUEFLOW
+	    || html_clueflow_get_style (HTML_CLUEFLOW (e->cursor->object->parent)) != HTML_CLUEFLOW_STYLE_PRE)
+		return;
+
+	html_engine_disable_selection (e);
+	position = e->cursor->position;
+
+	html_undo_level_begin (e->undo, "Indent PRE paragraph", "Reverse paragraph indentation");
+	html_engine_freeze (e);
+
+	last_space  = 0;
+	line_offset = 0;
+
+	html_cursor_beginning_of_paragraph (e->cursor, e);
+
+	flow = e->cursor->object->parent;
+	while (html_cursor_forward (e->cursor, e) && e->cursor->object->parent == flow) {
+		uc = html_cursor_get_current_char (e->cursor);
+		line_offset++;
+
+		if (uc == ' ') {
+				last_space = line_offset;
+		}
+		
+		if (line_offset >= LINE_LEN) {
+			if (last_space) {
+				html_cursor_backward_n (e->cursor, e, line_offset - last_space);
+
+				html_cursor_forward (e->cursor, e);
+				if ((uc = html_cursor_get_current_char (e->cursor))) {
+					html_engine_insert_empty_paragraph (e);
+					if (position >= e->cursor->position)
+						position++;
+				}
+			}	
+		}
+		if (!uc)
+			break;
+	}
+
+	html_cursor_jump_to_position (e->cursor, e, position);
+	html_engine_thaw (e);
+	html_undo_level_end (e->undo);
+}
+
+void
+html_engine_fill_pre_line (HTMLEngine *e)
+{
+	guint position;
+	guint line_offset;
+	guint last_space;
+	HTMLObject *flow;
+	unicode_char_t uc;
+
+	g_assert (e->cursor->object);
+	position = e->cursor->position;
+
+	if (HTML_OBJECT_TYPE (e->cursor->object->parent) != HTML_TYPE_CLUEFLOW
+	    || html_clueflow_get_style (HTML_CLUEFLOW (e->cursor->object->parent)) != HTML_CLUEFLOW_STYLE_PRE) {
+		return;
+	}
+
+	last_space  = 0;
+	line_offset = 0;
+
+	html_cursor_beginning_of_paragraph (e->cursor, e);
+
+	flow = e->cursor->object->parent;
+	while (html_cursor_forward (e->cursor, e) && (e->cursor->position < position - 1)) {
+		uc = html_cursor_get_current_char (e->cursor);
+		line_offset++;
+		
+		if (uc == ' ') {
+			last_space = line_offset;
+		}
+		
+		if (line_offset >= LINE_LEN) {
+			if (last_space) {
+				html_cursor_backward_n (e->cursor, e, line_offset - last_space);
+				
+				html_cursor_forward (e->cursor, e);
+				if ((uc = html_cursor_get_current_char (e->cursor))) {
+					html_engine_insert_empty_paragraph (e);
+					if (position >= e->cursor->position)
+						position++;
+
+					line_offset = 0;
+					last_space = 0;
+				}
+			}	
+		}
+		if (!uc)
+			break;
+	}
+	html_cursor_jump_to_position (e->cursor, e, position);
+}
+
+void
+html_engine_space_and_fill_line (HTMLEngine *e)
+{
+
+	g_assert (e->cursor->object);
+	html_undo_level_begin (e->undo, "insert and fill", "reverse insert and fill");
+
+
+	html_engine_disable_selection (e);
+	html_engine_freeze (e);
+	html_engine_insert_text (e, " ", 1);
+
+	html_engine_fill_pre_line (e);
+
+	html_engine_thaw (e);
+	html_undo_level_end (e->undo);
+}
+
+void
+html_engine_break_and_fill_line (HTMLEngine *e)
+{
+	html_undo_level_begin (e->undo, "break and fill", "reverse break and fill");
+
+	html_engine_disable_selection (e);
+	html_engine_freeze (e);
+
+	html_engine_fill_pre_line (e);
+
+	html_engine_insert_empty_paragraph (e);
+	html_engine_thaw (e);
+	html_undo_level_end (e->undo);
+}
