@@ -39,10 +39,12 @@ void                       html_engine_clipboard_clear        (HTMLEngine
 void                       html_engine_selection_push         (HTMLEngine                *e);
 void                       html_engine_selection_pop          (HTMLEngine                *e);
 void                       html_engine_cut_and_paste_begin    (HTMLEngine                *e,
-							       gchar                     *op_name);
+							       const gchar               *undo_op_name,
+							       const gchar               *redo_op_name);
 void                       html_engine_cut_and_paste_end      (HTMLEngine                *e);
 void                       html_engine_cut_and_paste          (HTMLEngine                *e,
-							       gchar                     *op_name,
+							       const gchar               *undo_op_name,
+							       const gchar               *redo_op_name,
 							       HTMLObjectForallFunc       iterator,
 							       gpointer                   data);
 void                       html_engine_spell_check_range      (HTMLEngine                *e,
@@ -60,6 +62,10 @@ HTMLObject                *html_engine_new_text_empty         (HTMLEngine
 gboolean                   html_engine_cursor_on_bop          (HTMLEngine                *e);
 guint                      html_engine_get_indent             (HTMLEngine                *e);
 void                       html_engine_indent_pre_paragraph   (HTMLEngine                *e);
+void                       html_engine_indent_pre_line        (HTMLEngine                *e);
+void                       html_engine_fill_pre_line          (HTMLEngine                *e);
+void                       html_engine_space_and_fill_line    (HTMLEngine                *e);
+void                       html_engine_break_and_fill_line    (HTMLEngine                *e);
 /*
   static (non instance) methods
 */
