@@ -20,8 +20,10 @@
 
 */
 
-#include "htmlform.h"
+#include <config.h>
 #include <string.h>
+#include "htmlform.h"
+#include "htmlengine.h"
 
 
 HTMLForm *
