@@ -21,15 +21,12 @@
 
 #ifndef _HTMLIFRAME_H_
 #define _HTMLIFRAME_H_
-#include "htmlobject.h"
+
 #include "htmlembedded.h"
 
 #define HTML_IFRAME(x) ((HTMLIFrame *)(x))
 #define HTML_IFRAME_CLASS(x) ((HTMLIFrameClass *)(x))
 
-typedef struct _HTMLIFrame HTMLIFrame;
-typedef struct _HTMLIFrameClass HTMLIFrameClass;
-
 struct _HTMLIFrame {
 	HTMLEmbedded embedded;
 	
