@@ -24,13 +24,17 @@
 
 #include <config.h>
 #include <string.h>
+#include "gtkhtmldebug.h"
 #include "htmlcolor.h"
 #include "htmlcolorset.h"
 #include "htmlengine.h"
+#include "htmlengine-edit.h"
 #include "htmlengine-save.h"
+#include "htmlimage.h"
 #include "htmlpainter.h"
 #include "htmlsearch.h"
 #include "htmltable.h"
+#include "htmltablepriv.h"
 #include "htmltablecell.h"
 
 
@@ -53,11 +57,11 @@
 HTMLTableClass html_table_class;
 static HTMLObjectClass *parent_class = NULL;
 
-static void alloc_cell (HTMLTable *table, gint r, gint c);
-static void set_cell   (HTMLTable *table, gint r, gint c, HTMLTableCell *cell);
 static void do_cspan   (HTMLTable *table, gint row, gint col, HTMLTableCell *cell);
 static void do_rspan   (HTMLTable *table, gint row);
 
+static void html_table_set_max_width (HTMLObject *o, HTMLPainter *painter, gint max_width);
+
 
 
 static inline gboolean
@@ -103,13 +107,431 @@ destroy (HTMLObject *o)
 }
 
 static void
-copy (HTMLObject *self,
-      HTMLObject *dest)
+copy_sized (HTMLObject *self, HTMLObject *dest, gint rows, gint cols)
 {
+	HTMLTable *d = HTML_TABLE (dest);
+	HTMLTable *s = HTML_TABLE (self);
+	gint r;
+
+	memcpy (dest, self, sizeof (HTMLTable));
 	(* HTML_OBJECT_CLASS (parent_class)->copy) (self, dest);
 
-	/* FIXME TODO*/
-	g_warning ("HTMLTable::copy is broken.");
+	d->bgColor     = s->bgColor ? gdk_color_copy (s->bgColor) : NULL;
+	d->caption     = s->caption ? HTML_CLUEV (html_object_dup (HTML_OBJECT (s->caption))) : NULL;
+
+	d->columnMin   = g_array_new (FALSE, FALSE, sizeof (gint));
+	d->columnFixed = g_array_new (FALSE, FALSE, sizeof (gint));
+	d->columnPref  = g_array_new (FALSE, FALSE, sizeof (gint));
+	d->columnOpt   = g_array_new (FALSE, FALSE, sizeof (gint));
+	d->rowHeights  = g_array_new (FALSE, FALSE, sizeof (gint));
+
+	d->totalCols = cols;
+	d->totalRows = rows;
+	d->allocRows = rows;
+
+	d->cells = g_new (HTMLTableCell **, rows);
+	for (r = 0; r < rows; r++)
+		d->cells [r] = g_new0 (HTMLTableCell *, cols);
+
+	dest->change = HTML_CHANGE_ALL;
+}
+
+static void
+copy (HTMLObject *self, HTMLObject *dest)
+{
+	copy_sized (self, dest, HTML_TABLE (self)->totalRows, HTML_TABLE (self)->totalCols);
+}
+
+static HTMLObject * op_copy (HTMLObject *self, HTMLEngine *e, GList *from, GList *to, guint *len);
+
+static HTMLObject *
+copy_as_leaf (HTMLObject *self, HTMLEngine *e, GList *from, GList *to, guint *len)
+{
+	if ((!from || GPOINTER_TO_INT (from->data) == 0)
+	    && (!to || GPOINTER_TO_INT (to->data) == html_object_get_length (self)))
+		return op_copy (self, e, NULL, NULL, len);
+	else
+		return html_engine_new_text_empty (e);
+}
+
+static HTMLObject *
+op_copy (HTMLObject *self, HTMLEngine *e, GList *from, GList *to, guint *len)
+{
+	HTMLTableCell *start, *end;
+	HTMLTable *nt, *t;
+	gint r, c, rows, cols, start_col;
+
+	g_assert (HTML_IS_TABLE (self));
+
+	if ((from || to)
+	    && (!from || !from->next)
+	    && (!to || !to->next))
+		return copy_as_leaf (self, e, from, to, len);
+
+	t  = HTML_TABLE (self);
+	nt = g_new0 (HTMLTable, 1);
+
+	start = HTML_TABLE_CELL (from ? from->data : html_object_head (self));
+	end   = HTML_TABLE_CELL (to   ? to->data   : html_object_tail (self));
+	rows  = end->row - start->row + 1;
+	cols  = end->row == start->row ? end->col - start->col + 1 : t->totalCols;
+
+	copy_sized (self, HTML_OBJECT (nt), rows, cols);
+
+	start_col = end->row == start->row ? start->col : 0;
+
+	printf ("cols: %d rows: %d\n", cols, rows);
+
+	*len = 0;
+	for (r = 0; r < rows; r++)
+		for (c = 0; c < cols; c++) {
+			HTMLTableCell *cell = t->cells [start->row + r][c + start_col];
+
+			if (!cell || (end->row != start->row
+				      && ((r == 0 && c < start_col) || (r == rows - 1 && c > end->col))))
+				continue;
+			if (cell->row == r + start->row && cell->col == c + start_col) {
+				HTMLTableCell *cell_copy;
+				cell_copy = HTML_TABLE_CELL
+					(html_object_op_copy (HTML_OBJECT (cell), e,
+							      html_object_get_bound_list (HTML_OBJECT (cell), from),
+							      html_object_get_bound_list (HTML_OBJECT (cell), to), len));
+				html_table_set_cell (nt, r, c, cell_copy);
+				html_table_cell_set_position (cell_copy, r, c);
+			} else
+				nt->cells [r][c] = nt->cells [cell->row - start->row][cell->col - start_col];
+		}
+
+	if (end->col - start_col < cols - 1)
+		do_cspan (nt, nt->totalRows - 1, end->col - start_col, nt->cells [nt->totalRows - 1][end->col - start_col]);
+
+	printf ("copy end: %d\n", *len);
+
+	return HTML_OBJECT (nt);
+}
+
+static guint
+get_recursive_length (HTMLObject *self)
+{
+	HTMLTable *t = HTML_TABLE (self);
+	guint r, c, len = 0;
+
+	for (r = 0; r < t->totalRows; r++)
+		for (c = 0; c < t->totalCols; c++)
+			if (t->cells [r][c] && t->cells [r][c]->row == r && t->cells [r][c]->col == c)
+				len += html_object_get_recursive_length (HTML_OBJECT (t->cells [r][c])) + 1;
+
+	/* if (len > 0)
+	   len --; */
+	len ++;
+	printf ("get_recursive_length %d\n", len);
+	return len;
+}
+
+static void
+remove_cell (HTMLTable *t, HTMLTableCell *cell)
+{
+	gint r, c;
+	printf ("remove cell %d,%d %d,%d %d,%d\n", cell->col, cell->row, cell->rspan, cell->cspan, t->totalCols, t->totalRows);
+	for (r = 0; r < cell->rspan && r + cell->row < t->totalRows; r++)
+		for (c = 0; c < cell->cspan && c + cell->col < t->totalCols; c++) {
+			printf ("clear: %d,%d (%d,%d) %d,%d\n", cell->row + r, cell->col + c, cell->rspan, cell->cspan, r, c);
+			t->cells [cell->row + r][cell->col + c] = NULL;
+		}
+	HTML_OBJECT (cell)->parent = NULL;
+}
+
+static HTMLObject *
+cut_whole (HTMLObject *self, guint *len)
+{
+	if (self->parent)
+		html_object_remove_child (self->parent, self);
+	*len = html_object_get_recursive_length (self) + 1;
+	printf ("removed whole table len: %d\n", *len);
+
+	return self;
+}
+
+static HTMLObject *
+cut_partial (HTMLObject *self, HTMLEngine *e, GList *from, GList *to, GList *left, GList *right, guint *len)
+{
+	HTMLObject *rv;
+
+	HTMLTableCell *start, *end, *cell;
+	HTMLTable *t, *nt;
+	gint r, c, rows, cols;
+	gint start_row, start_col, end_row, end_col;
+	gboolean shrink;
+
+	printf ("partial cut\n");
+
+	start = HTML_TABLE_CELL (from ? from->data : html_object_head (self));
+	end   = HTML_TABLE_CELL (to   ? to->data   : html_object_tail (self));
+
+	start_row = start->row;
+	start_col = start->col;
+	end_row   = end->row;
+	end_col   = end->col;
+
+	*len = 0;
+	t    = HTML_TABLE (self);
+	rows = end_row - start_row + 1;
+	cols = end_row == start_row ? end_col - start_col + 1 : t->totalCols;
+	rv   = HTML_OBJECT (g_new0 (HTMLTable, 1));
+	nt   = HTML_TABLE (rv);
+	copy_sized (self, rv, rows, cols);
+
+	/* remove selected and move it to new one */
+	for (r = start_row; r <= end_row; r++)
+		for (c = 0; c < t->totalCols; c++) {
+			cell = t->cells [r][c];
+			if (cell && (r > start_row || c >= start_col) && (r < end_row || c <= end_col)) {
+				HTMLTableCell *cell_cut;
+				gint row, col;
+
+				row = r - start_row;
+				col = end_row == start_row ? c - start_col : c;
+
+				cell_cut = HTML_TABLE_CELL
+					(html_object_op_cut
+					 (HTML_OBJECT (cell), e,
+					  html_object_get_bound_list (HTML_OBJECT (cell), from),
+					  html_object_get_bound_list (HTML_OBJECT (cell), to),
+					  left ? left->next : NULL, right ? right->next : NULL, len));
+				html_table_set_cell (nt, row, col, cell_cut);
+				html_table_cell_set_position (cell_cut, row, col);
+			}
+		}
+
+	shrink = start_row == 0 && end_row == t->totalRows - 1;
+	/* move remaining cells in old table */
+	if (start_col >= end_col)
+		start_row ++;
+	if (start_row != end_row)
+		for (r = end_row; r < t->totalRows; r ++)
+			for (c = 0; c < t->totalCols; c ++) {
+				HTMLTableCell *cell;
+
+				cell = t->cells [r][c];
+				if (cell && (r > end_row || c >= end_col)) {
+					gint new_c = shrink ? c - end_col + start_col + 1: c;
+					if (cell->row == r && cell->col == c)
+						html_table_cell_set_position (cell, r - end_row + start_row,
+									      new_c);
+					t->cells [r - end_row + start_row][new_c] = cell;
+					t->cells [r][c] = NULL;
+				}
+			}
+	if (shrink)
+		t->totalCols -= end_col - start_col - 1;
+	t->totalRows -= end_row - start_row;
+	printf ("removed partial table len: %d\n", *len);
+
+	return rv;
+}
+
+static HTMLObject *
+op_cut (HTMLObject *self, HTMLEngine *e, GList *from, GList *to, GList *left, GList *right, guint *len)
+{
+	if ((!from || !from->next) && (!to || !to->next))
+		return (*parent_class->op_cut) (self, e, from, to, left, right, len);
+
+	/* if ((to && !to->next && GPOINTER_TO_INT (to->data) == 0)
+	    || (from && !from->next && GPOINTER_TO_INT (from->data) == 1))
+	    return html_engine_new_text_empty (e); */
+
+	if (from || to)
+		return cut_partial (self, e, from, to, left, right, len);
+	else
+		return cut_whole (self, len);
+}
+
+static void
+split (HTMLObject *self, HTMLEngine *e, HTMLObject *child, gint offset, gint level, GList **left, GList **right)
+{
+	HTMLObject *dup;
+	HTMLTable *nt;
+	HTMLTable *t = HTML_TABLE (self);
+	HTMLTableCell *dup_cell;
+	HTMLTableCell *cell;
+	gint r, c, rows, cols, start_col, dup_row, dup_col;
+
+	if (*left == NULL && *right == NULL) {
+		(*parent_class->split)(self, e, child, offset, level, left, right);
+		return;
+	}
+
+	dup_cell  = HTML_TABLE_CELL ((*right)->data);
+	cell      = HTML_TABLE_CELL ((*left)->data);
+
+	printf ("before split\n");
+	printf ("-- self --\n");
+	gtk_html_debug_dump_tree_simple (self, 0);
+	printf ("-- child --\n");
+	gtk_html_debug_dump_tree_simple (child, 0);
+	printf ("-- child end --\n");
+
+	rows      = t->totalRows - dup_cell->row;
+	cols      = t->totalRows - 1 == dup_cell->row ? t->totalCols - dup_cell->col : t->totalCols;
+	start_col = t->totalRows - 1 == dup_cell->row ? dup_cell->col : 0;
+	dup_row   = dup_cell->row;
+	dup_col   = dup_cell->col;
+
+	dup = HTML_OBJECT (g_new0 (HTMLTable, 1));
+	copy_sized (self, dup, rows, cols);
+        nt = HTML_TABLE (dup);
+
+	for (r = 0; r < nt->totalRows; r ++)
+		for (c = 0; c < t->totalCols; c ++) {
+			HTMLTableCell *act_cell;
+
+			act_cell = t->cells [dup_row + r][start_col + c];
+			if (act_cell) {
+				if (act_cell->row == dup_row && act_cell->col == dup_col)
+					act_cell = dup_cell;
+				if ((act_cell->row == dup_row && act_cell->col >= dup_col)
+				    || act_cell->row > dup_row) {
+					html_table_set_cell (nt, r, c, act_cell);
+					if (dup_row + r == act_cell->row && start_col + c == act_cell->col)
+						html_table_cell_set_position (act_cell, r, c);
+				}
+				if (r || start_col + c > cell->col)
+					t->cells [dup_row + r][start_col + c] = NULL;
+
+			}
+					
+		}
+
+	t->totalRows = cell->row + 1;
+	if (cell->row == 0)
+		t->totalCols = cell->col + 1;
+
+	if (self->parent && HTML_OBJECT_TYPE (self->parent) != HTML_TYPE_TABLE)
+		html_clue_append_after (HTML_CLUE (self->parent), dup, self);
+
+	*left  = g_list_prepend (*left, self);
+	*right = g_list_prepend (*right, dup);
+
+	html_object_change_set (self, HTML_CHANGE_ALL);
+	html_object_change_set (dup,  HTML_CHANGE_ALL);
+
+	printf ("after split\n");
+	printf ("-- self --\n");
+	gtk_html_debug_dump_tree_simple (self,  0);
+	printf ("-- dup --\n");
+	gtk_html_debug_dump_tree_simple (dup, 0);
+	printf ("-- end split --\n");
+
+	level--;
+	if (level)
+		html_object_split (self->parent, e, dup, 0, level, left, right);
+}
+
+static gboolean
+merge (HTMLObject *self, HTMLObject *with, HTMLEngine *e, GList *left, GList *right)
+{
+	HTMLTable *t1 = HTML_TABLE (self);
+	HTMLTable *t2 = HTML_TABLE (with);
+	HTMLTableCell *c1 = HTML_TABLE_CELL (left->data);
+	HTMLTableCell *c2 = HTML_TABLE_CELL (right->data);
+	gint r, c, end_col, end_row, start_col;
+	end_col = 0;
+	
+	if (t1->specified_width != t2->specified_width
+	    || t1->spacing != t2->spacing
+	    || t1->padding != t2->padding
+	    || t1->border != t2->border
+	    || t1->capAlign != t2->capAlign
+	    || (t1->bgColor && t2->bgColor && !gdk_color_equal (t1->bgColor, t2->bgColor))
+	    || (t1->bgColor && !t2->bgColor) || (!t1->bgColor && t2->bgColor)
+	    || t1->bgPixmap != t2->bgPixmap)
+		return FALSE;
+
+	printf ("before merge\n");
+	printf ("left\n");
+	gtk_html_debug_dump_tree_simple (left->data, 0);
+	printf ("right\n");
+	gtk_html_debug_dump_tree_simple (right->data, 0);
+	printf ("-- self --\n");
+	gtk_html_debug_dump_tree_simple (self, 0);
+	printf ("-- with --\n");
+	gtk_html_debug_dump_tree_simple (with, 0);
+	printf ("-- end with --\n");
+
+	if (t1->totalRows == 1) {
+		HTMLTableCell *head = HTML_TABLE_CELL (html_object_head (HTML_OBJECT (t2)));
+
+		if (head->col + 1 > t1->totalCols) {
+			gint inc_col = head->col - t1->totalCols + 1;
+
+			html_table_alloc_cell (t1, 0, head->col + 1);
+			for (c = t1->totalCols - 1; c >= 0; c--) {
+				HTMLTableCell *cell = t1->cells [0][c];
+
+				if (cell) {
+					if (cell->col == c)
+						html_table_cell_set_position (cell, 0, c + inc_col);
+					html_table_set_cell (t1, 0, c + inc_col, cell);
+					t1->cells [0][c] = NULL;
+				}
+			}
+		}
+	}
+
+	for (c = 0; c < t1->totalCols; c++)
+		if (t1->cells [t1->totalRows - 1][c])
+			end_col = c;
+
+	end_row   = t1->totalRows - 1;
+	start_col = end_col;
+	if (t2->totalRows > 1) {
+		for (c = 0; c < MIN (end_col, t2->totalCols); c++)
+			if (t2->cells [0][c]) {
+				end_row ++;
+				start_col = 0;
+				break;
+			}
+	}
+
+	for (r = 0; r < t2->totalRows; r ++, end_row ++)
+		for (c = 0; c < t2->totalCols; c ++) {
+			HTMLTableCell *cell = t2->cells [r][c];
+
+			if (cell) {
+				if (cell->row == r && cell->col == c)
+					html_table_cell_set_position (cell, end_row, c);
+				html_table_alloc_cell (t1, end_row, c);
+				if (cell != c2 || c1->col != c2->col)
+					html_table_set_cell (t1, end_row, c, cell);
+				else
+					HTML_OBJECT (c2)->parent = NULL;
+
+				t2->cells [r][c] = NULL;
+			}
+		}
+
+	html_object_change_set (self, HTML_CHANGE_ALL);
+
+	printf ("after merge\n");
+	printf ("-- self --\n");
+	gtk_html_debug_dump_tree_simple (self, 0);
+	printf ("-- with --\n");
+	gtk_html_debug_dump_tree_simple (with, 0);
+	printf ("-- end merge --\n");
+
+	return TRUE;
+}
+
+static void
+remove_child (HTMLObject *self, HTMLObject *child)
+{
+	remove_cell (HTML_TABLE (self), HTML_TABLE_CELL (child));
+}
+
+static gboolean
+accepts_cursor (HTMLObject *self)
+{
+	return TRUE;
 }
 
 static gboolean
@@ -119,10 +541,7 @@ is_container (HTMLObject *object)
 }
 
 static void
-forall (HTMLObject *self,
-	HTMLEngine *e,
-	HTMLObjectForallFunc func,
-	gpointer data)
+forall (HTMLObject *self, HTMLEngine *e, HTMLObjectForallFunc func, gpointer data)
 {
 	HTMLTableCell *cell;
 	HTMLTable *table;
@@ -143,8 +562,6 @@ forall (HTMLObject *self,
 	(* func) (self, e, data);
 }
 
-
-
 static void
 previous_rows_do_cspan (HTMLTable *table, gint c)
 {
@@ -313,7 +730,7 @@ do_cspan (HTMLTable *table, gint row, gint col, HTMLTableCell *cell)
 	g_assert (cell->col <= col);
 
 	for (i=col - cell->col; i < cell->cspan && cell->col + i < table->totalCols; i++)
-		set_cell (table, row, cell->col + i, cell);
+		html_table_set_cell (table, row, cell->col + i, cell);
 }
 
 static void
@@ -323,7 +740,7 @@ prev_col_do_cspan (HTMLTable *table, gint row)
 
 	/* add previous column cell which has cspan > 1 */
 	while (table->col < table->totalCols && table->cells [row][table->col] != 0) {
-		alloc_cell (table, row, table->col + table->cells [row][table->col]->cspan);
+		html_table_alloc_cell (table, row, table->col + table->cells [row][table->col]->cspan);
 		do_cspan (table, row, table->col + 1, table->cells [row][table->col]);
 		table->col += (table->cells [row][table->col])->cspan;
 	}
@@ -340,13 +757,13 @@ do_rspan (HTMLTable *table, gint row)
 		if (table->cells [row - 1][i]
 		    && (table->cells [row - 1][i])->row + (table->cells [row - 1][i])->rspan
 		    > row) {
-			set_cell (table, table->row, i, table->cells [table->row - 1][i]);
+			html_table_set_cell (table, table->row, i, table->cells [table->row - 1][i]);
 			do_cspan (table, table->row, i + 1, table->cells [table->row -1][i]);
 		}
 }
 
-static void
-set_cell (HTMLTable *table, gint r, gint c, HTMLTableCell *cell)
+void
+html_table_set_cell (HTMLTable *table, gint r, gint c, HTMLTableCell *cell)
 {
 	if (!table->cells [r][c]) {
 		table->cells [r][c] = cell;
@@ -354,8 +771,8 @@ set_cell (HTMLTable *table, gint r, gint c, HTMLTableCell *cell)
 	}
 }
 
-static void
-alloc_cell (HTMLTable *table, gint r, gint c)
+void
+html_table_alloc_cell (HTMLTable *table, gint r, gint c)
 {
 	if (c >= table->totalCols)
 		inc_columns (table, c + 1 - table->totalCols);
@@ -410,7 +827,7 @@ calc_cells_size (HTMLTable *table, HTMLPainter *painter)
 }
 
 static void
-set_cells_position (HTMLTable *table, HTMLPainter *painter)
+html_table_set_cells_position (HTMLTable *table, HTMLPainter *painter)
 {
 	HTMLTableCell *cell;
 	gint r, c, rl, pixel_size = html_painter_get_pixel_size (painter);
@@ -442,9 +859,12 @@ calc_size (HTMLObject *o,
 	old_ascent  = o->ascent;
 	pixel_size  = html_painter_get_pixel_size (painter);
 
+	if (!table->columnOpt->data)
+		html_table_set_max_width (o, painter, o->max_width);
+
 	calc_cells_size (table, painter);
 	calc_row_heights (table, painter);
-	set_cells_position (table, painter);
+	html_table_set_cells_position (table, painter);
 
 	o->ascent = ROW_HEIGHT (table, table->totalRows) + pixel_size * table->border;
 	o->width  = COLUMN_OPT (table, table->totalCols) + pixel_size * table->border;
@@ -647,7 +1067,7 @@ calc_percentage_step (HTMLTable *table, gint *col_percent, gint *span_percent, g
 			cl = cell_end_col (table, cell);
 			if (col_percent [cl] - col_percent [c] < HTML_OBJECT (cell)->percent) {
 				gint cp, part, added, pleft, not_percented, np;
-
+				part = 0;
 				not_percented = 0;
 				for (cp = 0; cp < span; cp++)
 					if (!PERC (c + cp))
@@ -938,12 +1358,13 @@ divide_into_percented_all (HTMLTable *table, gint *col_percent, gint *max_size,
 #define CSPAN (MIN (cell->col + cell->cspan, table->totalCols) - cell->col - 1)
 
 static void
-set_cells_max_width (HTMLTable *table, HTMLPainter *painter, gint *max_size)
+html_table_set_cells_max_width (HTMLTable *table, HTMLPainter *painter, gint *max_size)
 {
 	HTMLTableCell *cell;
 	gint r, c, size, pixel_size = html_painter_get_pixel_size (painter);
 	gint border_extra = table->border ? 2 : 0;
-
+	size = 0;
+	
 	for (r = 0; r < table->totalRows; r++)
 		for (c = 0; c < table->totalCols; c++) {
 			cell = table->cells[r][c];
@@ -1042,7 +1463,7 @@ html_table_set_max_width (HTMLObject *o,
 			   max_width + glue - COLUMN_MIN (table, table->totalCols)
 			   - pixel_size * table->border);
 
-	set_cells_max_width (table, painter, max_size);
+	html_table_set_cells_max_width (table, painter, max_size);
 	set_columns_optimal_width (table, max_size, pixel_size);
 
 	/* printf ("max_width %d opt_width %d\n", o->max_width, COLUMN_OPT (table, table->totalCols) + ); */
@@ -1300,9 +1721,11 @@ append_selection_string (HTMLObject *self,
 			len = buffer->len;
 			html_object_append_selection_string (HTML_OBJECT (cell), buffer);
 			/* remove last \n from added text */
-			if (buffer->len != len && buffer->str [buffer->len-1] == '\n')
-				g_string_truncate (buffer, buffer->len - 1);
-			tab = TRUE;
+			if (buffer->len != len) {
+				tab = TRUE;
+				if (buffer->str [buffer->len-1] == '\n')
+					g_string_truncate (buffer, buffer->len - 1);
+			}
 		}
 		if (rlen + tabs == buffer->len)
 			g_string_truncate (buffer, rlen);
@@ -1544,6 +1967,11 @@ html_table_class_init (HTMLTableClass *klass,
 	html_object_class_init (object_class, type, object_size);
 
 	object_class->copy = copy;
+	object_class->op_copy = op_copy;
+	object_class->op_cut = op_cut;
+	object_class->split = split;
+	object_class->merge = merge;
+	object_class->accepts_cursor = accepts_cursor;
 	object_class->calc_size = calc_size;
 	object_class->draw = draw;
 	object_class->destroy = destroy;
@@ -1566,6 +1994,8 @@ html_table_class_init (HTMLTableClass *klass,
 	object_class->save_plain = save_plain;
 	object_class->check_page_split = check_page_split;
 	object_class->get_bg_color = get_bg_color;
+	object_class->get_recursive_length = get_recursive_length;
+	object_class->remove_child = remove_child;
 
 	parent_class = &html_object_class;
 }
@@ -1636,15 +2066,15 @@ html_table_new (gint width, gint percent,
 void
 html_table_add_cell (HTMLTable *table, HTMLTableCell *cell)
 {
-	alloc_cell (table, table->row, table->col);
+	html_table_alloc_cell (table, table->row, table->col);
 	prev_col_do_cspan (table, table->row);
 
 	/* look for first free cell */
 	while (table->cells [table->row][table->col] && table->col < table->totalCols)
 		table->col++;
 
-	alloc_cell (table, table->row, table->col);
-	set_cell (table, table->row, table->col, cell);
+	html_table_alloc_cell (table, table->row, table->col);
+	html_table_set_cell (table, table->row, table->col, cell);
 	html_table_cell_set_position (cell, table->row, table->col);
 	do_cspan(table, table->row, table->col, cell);
 }
