@@ -25,6 +25,7 @@
 
 #define HTML_TABLE(x) ((HTMLTable *)(x))
 #define HTML_TABLE_CLASS(x) ((HTMLTableClass *)(x))
+#define HTML_IS_TABLE(x) (HTML_CHECK_TYPE ((x), HTML_TYPE_TABLE))
 
 struct _HTMLTable {
 	HTMLObject object;
@@ -54,9 +55,6 @@ struct _HTMLTableClass {
 	HTMLObjectClass object_class;
 };
 
-
-HTMLTableClass html_table_class;
-
 
 void        html_table_type_init   (void);
 void        html_table_class_init  (HTMLTableClass *klass,
