@@ -82,7 +82,7 @@ get_tags (const HTMLText *text,
 
 	if (!html_color_equal (text->color, html_colorset_get_color (state->engine->settings->color_set, HTMLTextColor))
 	    && !html_color_equal (text->color, html_colorset_get_color (state->engine->settings->color_set, HTMLLinkColor))) {
-		opening_p += sprintf (opening_p, "<FONT COLOR=#%02x%02x%02x>",
+		opening_p += sprintf (opening_p, "<FONT COLOR=\"#%02x%02x%02x\">",
 				      text->color->color.red   >> 8,
 				      text->color->color.green >> 8,
 				      text->color->color.blue  >> 8);
@@ -91,7 +91,7 @@ get_tags (const HTMLText *text,
 
 	size = font_style & GTK_HTML_FONT_STYLE_SIZE_MASK;
 	if (size != 0) {
-		opening_p += sprintf (opening_p, "<FONT SIZE=%d>", size);
+		opening_p += sprintf (opening_p, "<FONT SIZE=\"%d\">", size);
 	}
 
 	if (font_style & GTK_HTML_FONT_STYLE_BOLD) {
@@ -132,7 +132,7 @@ get_tags (const HTMLText *text,
 	}
 
 	if (size != 0) {
-		ending_p += sprintf (ending_p, "</FONT SIZE=%d>", size);
+		ending_p += sprintf (ending_p, "</FONT SIZE=\"%d\">", size);
 	}
 
 
@@ -155,7 +155,7 @@ copy (HTMLObject *s,
 	dest->text = g_strdup (src->text);
 	dest->text_len      = src->text_len;
 	dest->font_style    = src->font_style;
-	dest->face          = src->face;
+	dest->face          = g_strdup (src->face);
 	dest->color         = src->color;
 	dest->select_start  = src->select_start;
 	dest->select_length = src->select_length;
@@ -399,7 +399,7 @@ op_cut (HTMLObject *self, HTMLEngine *e, GList *from, GList *to, GList *left, GL
 }
 
 static gboolean
-object_merge (HTMLObject *self, HTMLObject *with, HTMLEngine *e)
+object_merge (HTMLObject *self, HTMLObject *with, HTMLEngine *e, GList *left, GList *right)
 {
 	HTMLText *t1, *t2;
 	gchar *to_free;
@@ -410,6 +410,8 @@ object_merge (HTMLObject *self, HTMLObject *with, HTMLEngine *e)
 	if (t1->font_style != t2->font_style || t1->color != t2->color)
 		return FALSE;
 
+	printf ("merge '%s' '%s'\n", t1->text, t2->text);
+
 	merge_word_width (t1, t2, e->painter);
 
 	move_spell_errors (t2->spell_errors, 0, t1->text_len);
@@ -424,6 +426,7 @@ object_merge (HTMLObject *self, HTMLObject *with, HTMLEngine *e)
 	html_object_change_set (self, HTML_CHANGE_ALL);
 
 	html_text_request_word_width (t1, e->painter);
+	printf ("merged '%s'\n", t1->text);
 
 	/* printf ("--- after merge\n");
 	   debug_spell_errors (t1->spell_errors);
@@ -470,11 +473,11 @@ object_split (HTMLObject *self, HTMLEngine *e, HTMLObject *child, gint offset, g
 	html_clue_append_after (HTML_CLUE (self->parent), dup, self);
 
 	prev = self->prev;
-	if (t1->text_len == 0 && prev && html_object_merge (prev, self, e))
+	if (t1->text_len == 0 && prev && html_object_merge (prev, self, e, NULL, NULL))
 		self = prev;
 
 	if (t2->text_len == 0 && dup->next)
-		html_object_merge (dup, dup->next, e);
+		html_object_merge (dup, dup->next, e, NULL, NULL);
 
 	/* printf ("--- before split offset %d dup len %d\n", offset, HTML_TEXT (dup)->text_len);
 	   debug_spell_errors (HTML_TEXT (self)->spell_errors); */
@@ -574,7 +577,7 @@ calc_preferred_width (HTMLObject *self,
 
 	html_text_request_word_width (text, painter);
 
-	return text->word_width [text->words - 1];
+	return MAX (1, text->word_width [text->words - 1]);
 
 	/* return html_painter_calc_text_width (painter,
 					     text->text, text->text_len,
@@ -688,7 +691,7 @@ calc_min_width (HTMLObject *self, HTMLPainter *painter)
 			mw = w;
 	}
 
-	return mw;
+	return MAX (1, mw);
 }
 
 static void
@@ -1496,9 +1499,9 @@ struct _HTMLMagicInsertMatch
 typedef struct _HTMLMagicInsertMatch HTMLMagicInsertMatch;
 
 static HTMLMagicInsertMatch mim [] = {
-	{ "(news|telnet|nttp|file|http|ftp|https)://[-a-z0-9.]+[-a-z0-9](:[0-9]*)?(/[-a-z0-9_$.+!*(),;:@&=?/~#]*[^]'.}>) ,\"]*)?", NULL, NULL },
-	{ "www[-a-z0-9.]+[-a-z0-9](:[0-9]*)?(/[-A-Za-z0-9_$.+!*(),;:@&=?/~#]*[^]'.}>) ,\"]*)?", NULL, "http://" },
-	{ "ftp[-a-z0-9.]+[-a-z0-9](:[0-9]*)?(/[-A-Za-z0-9_$.+!*(),;:@&=?/~#]*[^]'.}>) ,\"]*)?", NULL, "ftp://" },
+	{ "(news|telnet|nttp|file|http|ftp|https)://[-a-z0-9.]+[-a-z0-9](:[0-9]*)?(/[-a-z0-9_$.+!*(),;:@&=?/~#]*[^]'.}>) ,?!;:\"])?", NULL, NULL },
+	{ "www[-a-z0-9.]+[-a-z0-9](:[0-9]*)?(/[-A-Za-z0-9_$.+!*(),;:@&=?/~#]*[^]'.}>) ,?!;:\"])?", NULL, "http://" },
+	{ "ftp[-a-z0-9.]+[-a-z0-9](:[0-9]*)?(/[-A-Za-z0-9_$.+!*(),;:@&=?/~#]*[^]'.}>) ,?!;:\"])?", NULL, "ftp://" },
 	{ "[-_a-z0-9.]+@[-_a-z0-9.]+", NULL, "mailto:" }
 };
 
@@ -1563,7 +1566,7 @@ html_text_magic_link (HTMLText *text, HTMLEngine *engine, guint offset)
 		return FALSE;
 	offset--;
 
-	html_undo_level_begin (engine->undo, "Magic link");
+	html_undo_level_begin (engine->undo, "Magic link", "Remove magic link");
 	saved_position = engine->cursor->position;
 
 	while (html_text_get_char (text, offset) != ' ' && html_text_get_char (text, offset) != ENTITY_NBSP && offset)
